@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -44,32 +44,49 @@ def isWin9x():
     else:
         return 0
 
-def runTests(args, tests, num = 0):
+def runTests(tests, protocol, host, debug, compress, threadPerConnection, num = 0):
+
+    args = ""
+    if protocol:
+        args += "--protocol " + protocol + " "
+    if host:
+        args += "--host " + host + " "
+    if debug:
+        args += "--debug "
+    if compress:
+        args += "--compress "
+    if threadPerConnection:
+        args += "--threadPerConnection "
+
+    if num > 0:
+        prefix = "[" + str(num) + "] *** "
+    else:
+        prefix = "*** "
 
     #
     # Run each of the tests.
     #
     for i in tests:
 
-	i = os.path.normpath(i)
-	dir = os.path.join(toplevel, "test", i)
+        i = os.path.normpath(i)
+        dir = os.path.join(toplevel, "test", i)
 
-	print
-	if(num > 0):
-	    print "[" + str(num) + "]",
-	print "*** running tests in " + dir,
-	print
+        print
+        print prefix + "running tests in " + dir
+
+        if len(args) > 0:
+            print prefix + "options: " + args
+
+        sys.stdout.flush()
 
         if isWin9x():
-	    status = os.system("python " + os.path.join(dir, "run.py " + args))
+            status = os.system("python " + os.path.join(dir, "run.py " + args))
         else:
             status = os.system(os.path.join(dir, "run.py " + args))
 
-	if status:
-	    if(num > 0):
-		print "[" + str(num) + "]",
-	    print "test in " + dir + " failed with exit status", status,
-	    sys.exit(status)
+        if status:
+            print prefix + "test in " + dir + " failed with exit status", status
+            sys.exit(status)
 
 #
 # List of all basic tests.
@@ -98,10 +115,17 @@ tests = [ \
     "Ice/custom", \
     "Ice/retry", \
     "Ice/timeout", \
+    "Ice/servantLocator", \
+    "Ice/threads", \
     "IceSSL/configuration", \
     "Freeze/dbmap", \
     "Freeze/complex", \
     "Freeze/evictor", \
+    "Freeze/oldevictor", \
+    "IceStorm/single", \
+    "IceStorm/federation", \
+    "IceStorm/federation2", \
+    "IceStorm/stress", \
     "FreezeScript/dbmap", \
     "FreezeScript/evictor", \
     "IceGrid/simple", \
@@ -109,17 +133,16 @@ tests = [ \
     "IceGrid/session", \
     "IceGrid/update", \
     "IceGrid/activation", \
+    "IceGrid/replicaGroup", \
     "IceGrid/replication", \
     "IceGrid/allocation", \
-    "IceStorm/single", \
-    "IceStorm/federation", \
-    "IceStorm/federation2", \
+    "IceGrid/distribution", \
     "Glacier2/router", \
     "Glacier2/attack", \
     "Glacier2/sessionControl", \
     "Glacier2/ssl", \
-    "Glacier2/filters", \
-    "Glacier2/addressFilter", \
+    "Glacier2/dynamicFiltering", \
+    "Glacier2/staticFiltering", \
     ]
 
 #
@@ -131,12 +154,12 @@ if isCygwin() == 0:
       ]
 
 def usage():
-    print "usage: " + sys.argv[0] + " -l -r <regex> -R <regex> --debug --protocol protocol --compress --host host --threadPerConnection"
+    print "usage: " + sys.argv[0] + " --all -l -r <regex> -R <regex> --debug --protocol tcp|ssl --compress --host host --threadPerConnection"
     sys.exit(2)
 
 try:
     opts, args = getopt.getopt(sys.argv[1:], "lr:R:", \
-    	["debug", "protocol=", "compress", "host=", "threadPerConnection"])
+        ["all", "debug", "protocol=", "compress", "host=", "threadPerConnection"])
 except getopt.GetoptError:
     usage()
 
@@ -144,27 +167,56 @@ if(args):
     usage()
 
 loop = 0
-args = ""
+all = 0
+protocol = None
+host = None
+debug = 0
+compress = 0
+threadPerConnection = 0
 for o, a in opts:
     if o == "-l":
         loop = 1
+    if o == "--all":
+        all = 1
     if o == "-r" or o == '-R':
-	import re
-	regexp = re.compile(a)
-	if o == '-r':
-	    def rematch(x): return regexp.search(x)
-	else:
-	    def rematch(x): return not regexp.search(x)
-	tests = filter(rematch, tests)
-    if o in ( "--protocol", "--host" ):
-	args += " " + o + " " + a
-    if o in ( "--debug", "--compress", "--threadPerConnection" ):
-	args += " " + o 
-    
+        import re
+        regexp = re.compile(a)
+        if o == '-r':
+            def rematch(x): return regexp.search(x)
+        else:
+            def rematch(x): return not regexp.search(x)
+        tests = filter(rematch, tests)
+    if o == "--protocol":
+        if a not in ( "ssl", "tcp"):
+            usage()
+        protocol = a
+    if o == "--host":
+        host = a
+    if o == "--debug":
+        debug = 1
+    if o == "--compress":
+        compress = 1
+    if o == "--threadPerConnection":
+        threadPerConnection = 1
+
+protocols = ["tcp", "ssl"]
+
 if loop:
     num = 1
     while 1:
-	runTests(args, tests, num)
-	num += 1
+        if all:
+            for protocol in protocols:
+                for compress in [0, 1]:
+                    for threadPerConnection in [0, 1]:
+                        runTests(tests, protocol, host, debug, compress, threadPerConnection, num)
+        else:
+            runTests(tests, protocol, host, debug, compress, threadPerConnection, num)
+        num += 1
 else:
-    runTests(args, tests)
+    if all:
+        for protocol in protocols:
+            for compress in [0, 1]:
+                for threadPerConnection in [0, 1]:
+                    runTests(tests, protocol, host, debug, compress, threadPerConnection)
+    else:
+        runTests(tests, protocol, host, debug, compress, threadPerConnection)
