@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -11,8 +11,11 @@
 import sys, os, TestUtil
 from threading import Thread
 
-debug = 0
-#debug = 1
+#
+# Set nreplicas to a number N to test replication with N replicas.
+#
+#nreplicas=0
+nreplicas=1
 
 for toplevel in [".", "..", "../..", "../../..", "../../../.."]:
     toplevel = os.path.normpath(toplevel)
@@ -21,71 +24,108 @@ for toplevel in [".", "..", "../..", "../../..", "../../../.."]:
 else:
     raise "can't find toplevel directory!"
 
-iceGridPort = "12010";
+iceGridPort = 12010;
 
 nodeOptions = r' --Ice.Warn.Connections=0' + \
               r' --IceGrid.Node.Endpoints=default' + \
               r' --IceGrid.Node.WaitTime=30' + \
               r' --Ice.ProgramName=icegridnode' + \
+              r' --IceGrid.Node.Trace.Replica=0' + \
               r' --IceGrid.Node.Trace.Activator=0' + \
               r' --IceGrid.Node.Trace.Adapter=0' + \
               r' --IceGrid.Node.Trace.Server=0' + \
+              r' --IceGrid.Node.ThreadPool.SizeWarn=0' + \
               r' --IceGrid.Node.PrintServersReady=node' + \
-	      r' --Ice.NullHandleAbort' + \
+              r' --Ice.NullHandleAbort' + \
               r' --Ice.ThreadPool.Server.Size=0' + \
               r' --Ice.ServerIdleTime=0';
 
 registryOptions = r' --Ice.Warn.Connections=0' + \
                   r' --IceGrid.Registry.PermissionsVerifier=IceGrid/NullPermissionsVerifier' + \
                   r' --IceGrid.Registry.AdminPermissionsVerifier=IceGrid/NullPermissionsVerifier' + \
+                  r' --IceGrid.Registry.SSLPermissionsVerifier=IceGrid/NullSSLPermissionsVerifier' + \
+                  r' --IceGrid.Registry.AdminSSLPermissionsVerifier=IceGrid/NullSSLPermissionsVerifier' + \
                   r' --IceGrid.Registry.Server.Endpoints=default' + \
                   r' --IceGrid.Registry.Internal.Endpoints=default' + \
-                  r' --IceGrid.Registry.Admin.Endpoints=default' + \
-                  r' --IceGrid.Registry.Server.Endpoints=default' + \
-                  r' --IceGrid.Registry.Internal.Endpoints=default' + \
-                  r' --IceGrid.Registry.Admin.Endpoints=default' + \
+                  r' --IceGrid.Registry.SessionManager.Endpoints=default' + \
                   r' --IceGrid.Registry.Trace.Session=0' + \
                   r' --IceGrid.Registry.Trace.Application=0' + \
                   r' --IceGrid.Registry.Trace.Node=0' + \
+                  r' --IceGrid.Registry.Trace.Replica=0' + \
                   r' --IceGrid.Registry.Trace.Adapter=0' + \
                   r' --IceGrid.Registry.Trace.Object=0' + \
                   r' --IceGrid.Registry.Trace.Server=0' + \
                   r' --IceGrid.Registry.Trace.Locator=0' + \
-                  r' --Ice.ThreadPool.Client.SizeWarn=10' + \
-                  r' --Ice.ThreadPool.Server.Size=0' + \
-                  r' --Ice.ServerIdleTime=0';
+                  r' --Ice.ThreadPool.Server.Size=0 ' + \
+                  r' --Ice.ThreadPool.Client.SizeWarn=0' + \
+                  r' --IceGrid.Registry.Client.ThreadPool.SizeWarn=0' + \
+                  r' --Ice.ServerIdleTime=0' + \
+                  r' --IceGrid.Registry.DefaultTemplates=' + os.path.join(toplevel, "config", "templates.xml")
+
+def getDefaultLocatorProperty():
+
+   i = 0
+   property = '--Ice.Default.Locator=IceGrid/Locator';
+   while i < nreplicas + 1:
+       property = property + ':default -p ' + str(iceGridPort + i)
+       i = i + 1
+
+   return '"' + property + '"'
 
 def startIceGridRegistry(testdir, dynamicRegistration = False):
 
-    global iceGridPort
-
     iceGrid = os.path.join(toplevel, "bin", "icegridregistry")
 
-    dataDir = os.path.join(testdir, "db", "registry")
-    if not os.path.exists(dataDir):
-        os.mkdir(dataDir)
-    else:
-        cleanDbDir(dataDir)
-
-    print "starting icegrid registry...",
-    command = iceGrid + TestUtil.clientServerOptions + ' --nowarn ' + registryOptions + \
-              r' --IceGrid.Registry.Client.Endpoints="default -p ' + iceGridPort + ' -t 30000" ' + \
-              r' --IceGrid.Registry.Data=' + dataDir + \
-              r' --IceGrid.Registry.DefaultTemplates=' + os.path.join(toplevel, "config", "templates.xml")
-
+    command = iceGrid + TestUtil.clientServerOptions + ' --nowarn ' + registryOptions
     if dynamicRegistration:
-        command += ' --IceGrid.Registry.DynamicRegistration'        
+        command += r' --IceGrid.Registry.DynamicRegistration'        
 
-    if debug:
-        print command
+    i = 0
+    while i < (nreplicas + 1):
 
-    (stdin, iceGridPipe) = os.popen4(command)
-    TestUtil.getServerPid(iceGridPipe)
-    TestUtil.getAdapterReady(iceGridPipe, True, 4)
+        if i == 0:
+            name = "registry"
+        else:
+            name = "replica-" + str(i)
+
+        dataDir = os.path.join(testdir, "db", name)
+        if not os.path.exists(dataDir):
+            os.mkdir(dataDir)
+        else:
+            cleanDbDir(dataDir)
+
+        print "starting icegrid " + name + "...",
+        cmd = command + \
+              r' --Ice.ProgramName=' + name + \
+              r' --IceGrid.Registry.Client.Endpoints="default -p ' + str(iceGridPort + i) + ' -t 30000" ' + \
+              r' --IceGrid.Registry.Data=' + dataDir
+
+        if i > 0:
+            cmd += r' --IceGrid.Registry.ReplicaName=' + name + ' ' + getDefaultLocatorProperty()
+
+        if TestUtil.debug:
+            print "(" + cmd + ")",
+
+        pipe = os.popen(cmd + " 2>&1")
+        TestUtil.getServerPid(pipe)
+        TestUtil.getAdapterReady(pipe, True, 4)
+        print "ok"
+
+        i = i + 1
+
+def shutdownIceGridRegistry():
+
+    i = nreplicas
+    while i > 0:
+        print "shutting down icegrid replica-" + str(i) + "...",
+        iceGridAdmin("registry shutdown replica-" + str(i))
+        print "ok"
+        i = i - 1
+
+    print "shutting down icegrid registry...",
+    iceGridAdmin("registry shutdown")
     print "ok"
 
-    return iceGridPipe
-
 def startIceGridNode(testdir):
 
     iceGrid = os.path.join(toplevel, "bin", "icegridnode")
@@ -96,20 +136,25 @@ def startIceGridNode(testdir):
     else:
         cleanDbDir(dataDir)
 
-    overrideOptions = '"' + TestUtil.clientServerOptions.replace("--", "") + \
-	              ' Ice.ServerIdleTime=0 Ice.PrintProcessId=0 Ice.PrintAdapterReady=0"'
+    overrideOptions = '"' 
+    for opt in TestUtil.clientServerOptions.split():
+        opt = opt.replace("--", "")
+        if opt.find("=") == -1:
+            opt += "=1"
+        overrideOptions += opt + " "
+    overrideOptions += ' Ice.ServerIdleTime=0 Ice.PrintProcessId=0 Ice.PrintAdapterReady=0"'
 
     print "starting icegrid node...",
     command = iceGrid + TestUtil.clientServerOptions + ' --nowarn ' + nodeOptions + \
-              r' "--Ice.Default.Locator=IceGrid/Locator:default -p ' + iceGridPort + '" ' + \
+              r' ' + getDefaultLocatorProperty() + \
               r' --IceGrid.Node.Data=' + dataDir + \
               r' --IceGrid.Node.Name=localnode' + \
               r' --IceGrid.Node.PropertiesOverride=' + overrideOptions
 
-    if debug:
-        print command
+    if TestUtil.debug:
+        print "(" + command + ")",
 
-    (stdin, iceGridPipe) = os.popen4(command)
+    iceGridPipe = os.popen(command + " 2>&1")
     TestUtil.getServerPid(iceGridPipe)
     TestUtil.getAdapterReady(iceGridPipe, False)
     TestUtil.waitServiceReady(iceGridPipe, 'node')
@@ -120,23 +165,23 @@ def startIceGridNode(testdir):
 
 def iceGridAdmin(cmd, ignoreFailure = False):
 
-    global iceGridPort
     iceGridAdmin = os.path.join(toplevel, "bin", "icegridadmin")
 
-    command = iceGridAdmin + TestUtil.clientOptions + \
-              r' --Ice.Default.Locator="IceGrid/Locator:default -p ' + iceGridPort + '" ' + \
-              r' -e "' + cmd + '" 2>&1'
+    user = r"admin1"
+    if cmd == "registry shutdown":
+        user = r"shutdown"
+    command = iceGridAdmin + TestUtil.clientOptions + ' ' + getDefaultLocatorProperty() + \
+              r" --IceGridAdmin.Username=" + user + " --IceGridAdmin.Password=test1 " + \
+              r' -e "' + cmd + '"'
 
-    if debug:
-        print command
+    if TestUtil.debug:
+        print "(" + command +")",
 
-    iceGridAdminPipe = os.popen(command)
+    iceGridAdminPipe = os.popen(command + " 2>&1")
 
     output = iceGridAdminPipe.readlines()
-        
     iceGridAdminStatus = TestUtil.closePipe(iceGridAdminPipe)
     if not ignoreFailure and iceGridAdminStatus:
-        print "icegridadmin command failed: " + cmd
         for line in output:
             print line
         TestUtil.killServers()
@@ -146,8 +191,6 @@ def iceGridAdmin(cmd, ignoreFailure = False):
     
 def killNodeServers():
     
-    global iceGridPort
-    
     for server in iceGridAdmin("server list"):
         server = server.strip()
         iceGridAdmin("server disable " + server, True)
@@ -155,14 +198,20 @@ def killNodeServers():
 
 def iceGridTest(name, application, additionalOptions = "", applicationOptions = ""):
 
+    if not TestUtil.isWin32() and os.getuid() == 0:
+        print
+        print "*** can't run test as root ***"
+        print
+        return
+
     testdir = os.path.join(toplevel, "test", name)
     client = os.path.join(testdir, "client")
 
-    clientOptions = " --Ice.Default.Locator=\"IceGrid/Locator:default -p " + iceGridPort + "\" " + additionalOptions
+    clientOptions = ' ' + getDefaultLocatorProperty() + ' ' + additionalOptions
 
-    iceGridRegistryPipe = startIceGridRegistry(testdir)
+    startIceGridRegistry(testdir)
     iceGridNodePipe = startIceGridNode(testdir)
-
+    
     if application != "":
         print "adding application...",
         iceGridAdmin('application add ' + os.path.join(testdir, application) + ' ' + \
@@ -170,12 +219,12 @@ def iceGridTest(name, application, additionalOptions = "", applicationOptions =
         print "ok"
 
     print "starting client...",
+    command = client + TestUtil.clientOptions + " " + clientOptions
 
-    command = client + TestUtil.clientOptions + " " + clientOptions + " 2>&1"
-    if debug:
-        print command
+    if TestUtil.debug:
+        print "(" + command +")",
 
-    clientPipe = os.popen(command)
+    clientPipe = os.popen(command + " 2>&1")
     print "ok"
 
     TestUtil.printOutputFromPipe(clientPipe)
@@ -198,9 +247,7 @@ def iceGridTest(name, application, additionalOptions = "", applicationOptions =
     print "shutting down icegrid node...",
     iceGridAdmin("node shutdown localnode")
     print "ok"
-    print "shutting down icegrid registry...",
-    iceGridAdmin("shutdown")
-    print "ok"
+    shutdownIceGridRegistry()
 
     TestUtil.joinServers()
 
@@ -213,26 +260,30 @@ def iceGridClientServerTest(name, additionalClientOptions, additionalServerOptio
     server = os.path.join(testdir, "server")
     client = os.path.join(testdir, "client")
 
-    clientOptions = "--Ice.Default.Locator=\"IceGrid/Locator:default -p " + iceGridPort + "\" " + \
-                    additionalClientOptions
-
-    serverOptions = "--Ice.Default.Locator=\"IceGrid/Locator:default -p " + iceGridPort + "\" " + \
-                    additionalServerOptions
+    clientOptions = getDefaultLocatorProperty() + ' ' + additionalClientOptions
+    serverOptions = getDefaultLocatorProperty() + ' ' + additionalServerOptions
     
-    iceGridRegistryPipe = startIceGridRegistry(testdir, True)
+    startIceGridRegistry(testdir, True)
 
     print "starting sever...",
-    serverPipe = os.popen(server + TestUtil.clientServerOptions + " " + serverOptions + " 2>&1")
+
+    command = server + TestUtil.clientServerOptions + " " + serverOptions
+
+    if TestUtil.debug:
+        print "(" + command +")",
+
+    serverPipe = os.popen(command + " 2>&1")
     TestUtil.getServerPid(serverPipe)
     TestUtil.getAdapterReady(serverPipe)
     print "ok"
 
     print "starting client...",
-    command = client + TestUtil.clientOptions + " " + clientOptions + " 2>&1"
-    if debug:
-        print command
+    command = client + TestUtil.clientOptions + " " + clientOptions
 
-    clientPipe = os.popen(command)
+    if TestUtil.debug:
+        print "(" + command +")",
+
+    clientPipe = os.popen(command + " 2>&1")
     print "ok"
 
     TestUtil.printOutputFromPipe(clientPipe)
@@ -242,9 +293,7 @@ def iceGridClientServerTest(name, additionalClientOptions, additionalServerOptio
         TestUtil.killServers()
         sys.exit(1)
 
-    print "shutting down icegrid registry...",
-    iceGridAdmin("shutdown")
-    print "ok"
+    shutdownIceGridRegistry()
 
     TestUtil.joinServers()
 
@@ -252,18 +301,14 @@ def iceGridClientServerTest(name, additionalClientOptions, additionalServerOptio
         sys.exit(1)
 
 def cleanDbDir(path):
-    
-    files = os.listdir(path)
-    for filename in files:
-        if filename != "CVS" and filename != ".dummy":
-            fullpath = os.path.join(path, filename);
-            if os.path.isdir(fullpath):
-                cleanDbDir(fullpath)
-		try:
-		    os.rmdir(fullpath)
-		except OSError:
-		    # This might fail if the directory is empty (because it itself is
-		    # a CVS directory).
-		    pass
-            else:
-                os.remove(fullpath)
+    for filename in [ os.path.join(path, f) for f in os.listdir(path) if f != ".gitignore"]:
+        if os.path.isdir(filename):
+            cleanDbDir(filename)
+            try:
+                os.rmdir(filename)
+            except OSError:
+                # This might fail if the directory is empty (because
+                # it itself contains a .gitignore file.
+                pass
+        else:
+            os.remove(filename)
