@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -12,10 +12,12 @@ import fileinput, re, string
 
 previous = ""
 
+commentre = re.compile("^#")
+
 for line in fileinput.input():
     line = line.strip()
 
-    if re.compile("^#").search(line, 0):
+    if commentre.search(line, 0):
         continue;
 
     line = string.replace(line, ".o:", "$(OBJEXT):")
