@@ -1,30 +1,19 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <ChatSessionI.h>
-#include <Ice/Application.h>
+#include <Ice/Ice.h>
 #include <Glacier2/PermissionsVerifier.h>
+#include <ChatSessionI.h>
 
 using namespace std;
 using namespace Demo;
 
-class DummyPermissionsVerifierI : public Glacier2::PermissionsVerifier
-{
-public:
-
-    virtual bool
-    checkPermissions(const string& userId, const string& passwd, string&, const Ice::Current&) const
-    {
-	return true;
-    }
-};
-
 class ChatSessionManagerI : public Glacier2::SessionManager
 {
 public:
@@ -32,7 +21,7 @@ public:
     virtual Glacier2::SessionPrx
     create(const string& userId, const Glacier2::SessionControlPrx&, const Ice::Current& current)
     {
-	return Glacier2::SessionPrx::uncheckedCast(current.adapter->addWithUUID(new ChatSessionI(userId)));
+        return Glacier2::SessionPrx::uncheckedCast(current.adapter->addWithUUID(new ChatSessionI(userId)));
     }
 };
 
@@ -43,14 +32,13 @@ public:
     virtual int
     run(int, char*[])
     {
-	Ice::ObjectAdapterPtr adapter = communicator()->createObjectAdapter("ChatServer");
-	
-	adapter->add(new DummyPermissionsVerifierI, communicator()->stringToIdentity("verifier"));
-	adapter->add(new ChatSessionManagerI, communicator()->stringToIdentity("ChatSessionManager"));
-	adapter->activate();
-	communicator()->waitForShutdown();
-	
-	return EXIT_SUCCESS;
+        Ice::ObjectAdapterPtr adapter = communicator()->createObjectAdapter("ChatServer");
+        
+        adapter->add(new ChatSessionManagerI, communicator()->stringToIdentity("ChatSessionManager"));
+        adapter->activate();
+        communicator()->waitForShutdown();
+        
+        return EXIT_SUCCESS;
     }
 };
 
