@@ -1,6 +1,6 @@
 //{{NO_DEPENDENCIES}}
-// Microsoft Visual C++ generated include file.
-// Used by Hello.rc
+// Microsoft Developer Studio generated include file.
+// Used by HelloClient.rc
 //
 #define IDD_HELLOCLIENT_DIALOG          101
 #define IDR_MAINFRAME                   128
@@ -12,6 +12,7 @@
 #define IDC_FLUSH                       1005
 #define IDC_SHUTDOWN                    1006
 #define IDC_STATUSBAR                   1007
+#define IDC_DELAY                       1008
 
 // Next default values for new objects
 // 
