@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -8,6 +8,7 @@
 // **********************************************************************
 
 #include "stdafx.h"
+#include <IceUtil/IceUtil.h>
 #include <HelloI.h>
 
 using namespace std;
@@ -18,8 +19,12 @@ HelloI::HelloI(const LogIPtr& log, CDialog* dialog) :
 }
 
 void
-HelloI::sayHello(const Ice::Current&) const
+HelloI::sayHello(int delay, const Ice::Current&) const
 {
+    if(delay != 0)
+    {
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(delay));
+    }
     _log->message("Hello World!");
 }
 
