@@ -1,14 +1,14 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+#include <Ice/Ice.h>
 #include <CallbackI.h>
-#include <Ice/Application.h>
 
 using namespace std;
 using namespace Demo;
@@ -38,12 +38,12 @@ CallbackServer::run(int argc, char* argv[])
     sender->start();
     try
     {
-	communicator()->waitForShutdown();
+        communicator()->waitForShutdown();
     }
     catch(...)
     {
-	sender->destroy();
-	throw;
+        sender->destroy();
+        throw;
     }
     sender->destroy();
 
