@@ -1,15 +1,15 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <EchoI.h>
+#include <IceUtil/IceUtil.h>
 #include <Ice/Ice.h>
-#include <IceUtil/StringUtil.h>
+#include <EchoI.h>
 
 using namespace std;
 
