@@ -1,20 +1,25 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <HelloI.h>
+#include <IceUtil/IceUtil.h>
 #include <Ice/Ice.h>
+#include <HelloI.h>
 
 using namespace std;
 
 void
-HelloI::sayHello(const Ice::Current&) const
+HelloI::sayHello(int delay, const Ice::Current&) const
 {
+    if(delay != 0)
+    {
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(delay));
+    }
     cout << "Hello World!" << endl;
 }
 
