@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -46,5 +46,5 @@ HelloServiceI::start(const string& name, const Ice::CommunicatorPtr& communicato
 void
 HelloServiceI::stop()
 {
-    _adapter->deactivate();
+    _adapter->destroy();
 }
