@@ -7,7 +7,7 @@
 #include <map>
 #include <list>
 
-class EvictorBase : virtual public Ice::ServantLocator
+class EvictorBase : public Ice::ServantLocator
 {
 public:
 
@@ -17,6 +17,8 @@ public:
     virtual void finished(const Ice::Current&, const Ice::ObjectPtr&, const Ice::LocalObjectPtr&);
     virtual void deactivate(const std::string&);
 
+protected:
+
     virtual Ice::ObjectPtr add(const Ice::Current&, Ice::LocalObjectPtr&) = 0;
     virtual void evict(const Ice::ObjectPtr&, const Ice::LocalObjectPtr&) = 0;
 
@@ -30,18 +32,12 @@ private:
 
     struct EvictorEntry : public Ice::LocalObject
     {
-	Ice::ObjectPtr servant;
-	Ice::LocalObjectPtr userCookie;
-	EvictorQueue::iterator pos;
-	int useCount;
+        Ice::ObjectPtr servant;
+        Ice::LocalObjectPtr userCookie;
+        EvictorQueue::iterator queuePos;
+        int useCount;
     };
 
-    struct EvictorCookie : public Ice::LocalObject
-    {
-	EvictorEntryPtr entry;
-    };
-    typedef IceUtil::Handle<EvictorCookie> EvictorCookiePtr;
-
     EvictorMap _map;
     EvictorQueue _queue;
     Ice::Int _size;
