@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,33 +19,33 @@ namespace Filesystem {
 
     class NodeI : virtual public Node {
     public:
-	virtual std::string name(const Ice::Current &) const;
-	NodeI(const Ice::CommunicatorPtr &, const std::string &, const DirectoryIPtr & parent);
-	static Ice::ObjectAdapterPtr _adapter;
+        virtual std::string name(const Ice::Current &);
+        NodeI(const Ice::CommunicatorPtr &, const std::string &, const DirectoryIPtr & parent);
+        static Ice::ObjectAdapterPtr _adapter;
     private:
-	const std::string _name;
-	DirectoryIPtr _parent;
+        std::string _name;
+        DirectoryIPtr _parent;
     };
 
     class FileI : virtual public File,
-		  virtual public NodeI {
+                  virtual public NodeI {
     public:
-	virtual Lines read(const Ice::Current &) const;
-	virtual void write(const Lines &,
-			   const Ice::Current &);
-	FileI(const Ice::CommunicatorPtr &, const std::string &, const DirectoryIPtr &);
+        virtual Lines read(const Ice::Current &);
+        virtual void write(const Lines &,
+                           const Ice::Current &);
+        FileI(const Ice::CommunicatorPtr &, const std::string &, const DirectoryIPtr &);
     private:
-	Lines _lines;
+        Lines _lines;
     };
 
     class DirectoryI : virtual public Directory,
-		       virtual public NodeI {
+                       virtual public NodeI {
     public:
-	virtual NodeSeq list(const Ice::Current &) const;
+        virtual NodeSeq list(const Ice::Current &);
         DirectoryI(const Ice::CommunicatorPtr &, const std::string &, const DirectoryIPtr &);
-	void addChild(NodePrx child);
+        void addChild(NodePrx child);
     private:
-	NodeSeq _contents;
+        NodeSeq _contents;
     };
 }
 
