@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -24,7 +24,8 @@ class ICE_API Direct : private IceUtil::noncopyable
 public:
 
     Direct(const Ice::Current&);
-    ~Direct();
+    
+    void destroy();
 
     const Ice::ObjectPtr& servant();    
 
