@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,19 +10,13 @@
 #ifndef ICE_GC_SHARED_H
 #define ICE_GC_SHARED_H
 
-#include <IceUtil/Config.h>
-#include <Ice/GCRecMutex.h>
+#include <Ice/Config.h>
 #include <Ice/GCCountMap.h>
-#include <set>
 
 namespace IceInternal
 {
 
 class GC;
-class GCShared;
-
-typedef std::set<GCShared*> GCObjectSet;
-extern ICE_API GCObjectSet gcObjects; // Set of pointers to all existing classes with class data members.
 
 class ICE_API GCShared
 {
@@ -50,16 +44,19 @@ public:
 
     int __getRefUnsafe() const
     {
-	return _ref;
+        return _ref;
     }
 
     void __decRefUnsafe()
     {
-	--_ref;
+        --_ref;
     }
 
 protected:
 
+    void __gcIncRef();
+    void __gcDecRef();
+
     int _ref;
     bool _noDelete;
 
