@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,9 +10,6 @@
 #ifndef ICE_ICE_H
 #define ICE_ICE_H
 
-#include <Ice/GCRecMutex.h>
-#include <Ice/GCShared.h>
-#include <Ice/GC.h>
 #include <Ice/Initialize.h>
 #include <Ice/LocalException.h>
 #include <Ice/Properties.h>
@@ -32,4 +29,7 @@
 #include <Ice/Connection.h>
 #include <Ice/Functional.h>
 #include <Ice/Stream.h>
+#include <Ice/ImplicitContext.h>
+#include <Ice/Locator.h>
+#include <Ice/Router.h>
 #endif
