@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -91,33 +91,84 @@ public:
 
     ::std::string ice_toString() const;
 
-    bool ice_isA(const ::std::string&);
-    bool ice_isA(const ::std::string&, const ::Ice::Context&);
-    void ice_ping();
-    void ice_ping(const ::Ice::Context&);
-    ::std::vector< ::std::string> ice_ids();
-    ::std::vector< ::std::string> ice_ids(const ::Ice::Context&);
-    ::std::string ice_id();
-    ::std::string ice_id(const ::Ice::Context&);
+    bool ice_isA(const ::std::string& typeId)
+    {
+        return ice_isA(typeId, 0);
+    }
+    bool ice_isA(const ::std::string& typeId, const ::Ice::Context& context)
+    {
+        return ice_isA(typeId, &context);
+    }
+    
+    void ice_ping()
+    {
+        ice_ping(0);
+    }
+    void ice_ping(const ::Ice::Context& context)
+    {
+        ice_ping(&context);
+    }
+    
+    ::std::vector< ::std::string> ice_ids()
+    {
+        return ice_ids(0);
+    }
+    ::std::vector< ::std::string> ice_ids(const ::Ice::Context& context)
+    {
+        return ice_ids(&context);
+    }
+    
+    ::std::string ice_id()
+    {
+        return ice_id(0);
+    }
+    ::std::string ice_id(const ::Ice::Context& context)
+    {
+        return ice_id(&context);
+    }
 
     // Returns true if ok, false if user exception.
-    bool ice_invoke(const ::std::string&, ::Ice::OperationMode, const ::std::vector< ::Ice::Byte>&,
-	            ::std::vector< ::Ice::Byte>&);
-    bool ice_invoke(const ::std::string&, ::Ice::OperationMode, const ::std::vector< ::Ice::Byte>&,
-	            ::std::vector< ::Ice::Byte>&, const ::Ice::Context&);
-    bool ice_invoke(const ::std::string&, ::Ice::OperationMode, 
-    		    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&, ::std::vector< ::Ice::Byte>&);
-    bool ice_invoke(const ::std::string&, ::Ice::OperationMode, 
-    		    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&, ::std::vector< ::Ice::Byte>&,
-		    const ::Ice::Context&);
+    bool ice_invoke(const ::std::string& operation, 
+                    ::Ice::OperationMode mode, 
+                    const ::std::vector< ::Ice::Byte>& inParams,
+                    ::std::vector< ::Ice::Byte>& outParams)
+    {
+        const ::Ice::Context* context = 0;
+        return ice_invoke(operation, mode, inParams, outParams, context);
+    }
+    bool ice_invoke(const ::std::string& operation, 
+                    ::Ice::OperationMode mode, 
+                    const ::std::vector< ::Ice::Byte>& inParams,
+                    ::std::vector< ::Ice::Byte>& outParams,
+                    const ::Ice::Context& context)
+    {
+        return ice_invoke(operation, mode, inParams, outParams, &context);
+    }
+    bool ice_invoke(const ::std::string& operation, 
+                    ::Ice::OperationMode mode, 
+                    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>& inParams, 
+                    ::std::vector< ::Ice::Byte>& outParams)
+    {
+        const ::Ice::Context* context = 0;
+        return ice_invoke(operation, mode, inParams, outParams, context);
+    }
+    bool ice_invoke(const ::std::string& operation, 
+                    ::Ice::OperationMode mode,
+                    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>& inParams,
+                    ::std::vector< ::Ice::Byte>& outParams,
+                    const ::Ice::Context& context)
+    {
+        return ice_invoke(operation, mode, inParams, outParams, &context);
+    }
+
     void ice_invoke_async(const ::Ice::AMI_Object_ice_invokePtr&, const ::std::string&, ::Ice::OperationMode,
-    			  const ::std::vector< ::Ice::Byte>&);
+                          const ::std::vector< ::Ice::Byte>&);
     void ice_invoke_async(const ::Ice::AMI_Object_ice_invokePtr&, const ::std::string&, ::Ice::OperationMode,
-    			  const ::std::vector< ::Ice::Byte>&, const ::Ice::Context&);
+                          const ::std::vector< ::Ice::Byte>&, const ::Ice::Context&);
     void ice_invoke_async(const ::Ice::AMI_Array_Object_ice_invokePtr&, const ::std::string&, ::Ice::OperationMode,
-			  const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&);
+                          const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&);
     void ice_invoke_async(const ::Ice::AMI_Array_Object_ice_invokePtr&, const ::std::string&, ::Ice::OperationMode, 
-			  const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&, const ::Ice::Context&);
+                          const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&, const ::Ice::Context&);
 
     ::Ice::Identity ice_getIdentity() const;
     ICE_DEPRECATED_API ::Ice::ObjectPrx ice_newIdentity(const ::Ice::Identity&) const;
@@ -126,7 +177,7 @@ public:
     ::Ice::Context ice_getContext() const;
     ICE_DEPRECATED_API ::Ice::ObjectPrx ice_newContext(const ::Ice::Context&) const;
     ::Ice::ObjectPrx ice_context(const ::Ice::Context&) const;
-    ::Ice::ObjectPrx ice_defaultContext() const;
+    ICE_DEPRECATED_API ::Ice::ObjectPrx ice_defaultContext() const;
 
     const ::std::string& ice_getFacet() const;
     ICE_DEPRECATED_API ::Ice::ObjectPrx ice_newFacet(const ::std::string&) const;
@@ -152,6 +203,9 @@ public:
     bool ice_isSecure() const;
     ::Ice::ObjectPrx ice_secure(bool) const;
 
+    bool ice_isPreferSecure() const;
+    ::Ice::ObjectPrx ice_preferSecure(bool) const;
+
     ::Ice::RouterPrx ice_getRouter() const;
     ::Ice::ObjectPrx ice_router(const ::Ice::RouterPrx&) const;
 
@@ -177,14 +231,21 @@ public:
     ::Ice::ObjectPrx ice_timeout(int) const;
     ::Ice::ObjectPrx ice_connectionId(const ::std::string&) const;
 
+    bool ice_isThreadPerConnection() const;
+    ::Ice::ObjectPrx ice_threadPerConnection(bool) const;
+
     ICE_DEPRECATED_API ::Ice::ConnectionPtr ice_connection();
     ::Ice::ConnectionPtr ice_getConnection();
+    ::Ice::ConnectionPtr ice_getCachedConnection() const;
 
     ::IceInternal::ReferencePtr __reference() const;
     void __copyFrom(const ::Ice::ObjectPrx&);
-    void __handleException(const ::Ice::LocalException&, int&);
-    void __handleExceptionWrapper(const ::IceInternal::LocalExceptionWrapper&);
-    void __handleExceptionWrapperRelaxed(const ::IceInternal::LocalExceptionWrapper&, int&);
+    void __handleException(const ::IceInternal::Handle< ::IceDelegate::Ice::Object>&, 
+                           const ::Ice::LocalException&, int&);
+    void __handleExceptionWrapper(const ::IceInternal::Handle< ::IceDelegate::Ice::Object>&, 
+                                  const ::IceInternal::LocalExceptionWrapper&);
+    void __handleExceptionWrapperRelaxed(const ::IceInternal::Handle< ::IceDelegate::Ice::Object>&,
+                                         const ::IceInternal::LocalExceptionWrapper&, int&);
     void __checkTwowayOnly(const char*) const;
     void __checkTwowayOnly(const ::std::string&) const;
 
@@ -195,10 +256,25 @@ protected:
     virtual ::IceInternal::Handle< ::IceDelegateM::Ice::Object> __createDelegateM();
     virtual ::IceInternal::Handle< ::IceDelegateD::Ice::Object> __createDelegateD();
 
-    const ::Ice::Context& __defaultContext() const;
-
 private:
 
+    bool ice_isA(const ::std::string&, const ::Ice::Context*);
+    void ice_ping(const ::Ice::Context*);
+    ::std::vector< ::std::string> ice_ids(const ::Ice::Context*);
+    ::std::string ice_id(const ::Ice::Context*);
+
+   
+    bool ice_invoke(const ::std::string&, 
+                    ::Ice::OperationMode, 
+                    const ::std::vector< ::Ice::Byte>&,
+                    ::std::vector< ::Ice::Byte>&,
+                    const ::Ice::Context*);
+    bool ice_invoke(const ::std::string&, 
+                    ::Ice::OperationMode,
+                    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
+                    ::std::vector< ::Ice::Byte>&,
+                    const ::Ice::Context*);
+  
     void setup(const ::IceInternal::ReferencePtr&);
     friend class ::IceInternal::ProxyFactory;
 
@@ -217,13 +293,13 @@ class ICE_API Object : public ::IceUtil::Shared
 {
 public:
 
-    virtual bool ice_isA(const ::std::string&, const ::Ice::Context&) = 0;
-    virtual void ice_ping(const ::Ice::Context&) = 0;
-    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context&) = 0;
-    virtual ::std::string ice_id(const ::Ice::Context&) = 0;
+    virtual bool ice_isA(const ::std::string&, const ::Ice::Context*) = 0;
+    virtual void ice_ping(const ::Ice::Context*) = 0;
+    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context*) = 0;
+    virtual ::std::string ice_id(const ::Ice::Context*) = 0;
     virtual bool ice_invoke(const ::std::string&, ::Ice::OperationMode,
-    			    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
-			    ::std::vector< ::Ice::Byte>&, const ::Ice::Context&) = 0;
+                            const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
+                            ::std::vector< ::Ice::Byte>&, const ::Ice::Context*) = 0;
 
     virtual ::Ice::ConnectionIPtr __getConnection(bool&) const = 0;
 };
@@ -239,13 +315,13 @@ public:
 
     virtual ~Object();
 
-    virtual bool ice_isA(const ::std::string&, const ::Ice::Context&);
-    virtual void ice_ping(const ::Ice::Context&);
-    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context&);
-    virtual ::std::string ice_id(const ::Ice::Context&);
+    virtual bool ice_isA(const ::std::string&, const ::Ice::Context*);
+    virtual void ice_ping(const ::Ice::Context*);
+    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context*);
+    virtual ::std::string ice_id(const ::Ice::Context*);
     virtual bool ice_invoke(const ::std::string&, ::Ice::OperationMode, 
-    			    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
-			    ::std::vector< ::Ice::Byte>&, const ::Ice::Context&);
+                            const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
+                            ::std::vector< ::Ice::Byte>&, const ::Ice::Context*);
 
     virtual ::Ice::ConnectionIPtr __getConnection(bool&) const;
 
@@ -272,13 +348,13 @@ class ICE_API Object : virtual public ::IceDelegate::Ice::Object
 {
 public:
 
-    virtual bool ice_isA(const ::std::string&, const ::Ice::Context&);
-    virtual void ice_ping(const ::Ice::Context&);
-    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context&);
-    virtual ::std::string ice_id(const ::Ice::Context&);
+    virtual bool ice_isA(const ::std::string&, const ::Ice::Context*);
+    virtual void ice_ping(const ::Ice::Context*);
+    virtual ::std::vector< ::std::string> ice_ids(const ::Ice::Context*);
+    virtual ::std::string ice_id(const ::Ice::Context*);
     virtual bool ice_invoke(const ::std::string&, ::Ice::OperationMode,
-    			    const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
-			    ::std::vector< ::Ice::Byte>&, const ::Ice::Context&);
+                            const ::std::pair<const ::Ice::Byte*, const ::Ice::Byte*>&,
+                            ::std::vector< ::Ice::Byte>&, const ::Ice::Context*);
 
     virtual ::Ice::ConnectionIPtr __getConnection(bool&) const;
 
@@ -289,7 +365,7 @@ protected:
     ::IceInternal::ReferencePtr __reference;
     ::Ice::ObjectAdapterPtr __adapter;
 
-    void __initCurrent(::Ice::Current&, const ::std::string&, ::Ice::OperationMode, const ::Ice::Context&);
+    void __initCurrent(::Ice::Current&, const ::std::string&, ::Ice::OperationMode, const ::Ice::Context*);
 
 private:
 
@@ -312,7 +388,7 @@ struct ProxyIdentityLess : std::binary_function<bool, ObjectPrx&, ObjectPrx&>
 {
     bool operator()(const ObjectPrx& lhs, const ObjectPrx& rhs) const
     {
-	return proxyIdentityLess(lhs, rhs);
+        return proxyIdentityLess(lhs, rhs);
     }
 };
 
@@ -320,7 +396,7 @@ struct ProxyIdentityEqual : std::binary_function<bool, ObjectPrx&, ObjectPrx&>
 {
     bool operator()(const ObjectPrx& lhs, const ObjectPrx& rhs) const
     {
-	return proxyIdentityEqual(lhs, rhs);
+        return proxyIdentityEqual(lhs, rhs);
     }
 };
 
@@ -328,7 +404,7 @@ struct ProxyIdentityAndFacetLess : std::binary_function<bool, ObjectPrx&, Object
 {
     bool operator()(const ObjectPrx& lhs, const ObjectPrx& rhs) const
     {
-	return proxyIdentityAndFacetLess(lhs, rhs);
+        return proxyIdentityAndFacetLess(lhs, rhs);
     }
 };
 
@@ -336,7 +412,7 @@ struct ProxyIdentityAndFacetEqual : std::binary_function<bool, ObjectPrx&, Objec
 {
     bool operator()(const ObjectPrx& lhs, const ObjectPrx& rhs) const
     {
-	return proxyIdentityAndFacetEqual(lhs, rhs);
+        return proxyIdentityAndFacetEqual(lhs, rhs);
     }
 };
 
@@ -348,42 +424,22 @@ namespace IceInternal
 //
 // checkedCast and uncheckedCast functions without facet:
 //
-
-//
-// Out of line implementations
-//
 template<typename P> P 
-checkedCastImpl(const ::Ice::ObjectPrx& b)
+checkedCastImpl(const ::Ice::ObjectPrx& b, const ::Ice::Context* context)
 {
     P d = 0;
     if(b.get())
     {
-	typedef typename P::element_type T;
+        typedef typename P::element_type T;
 
-	d = dynamic_cast<T*>(b.get());
-	if(!d && b->ice_isA(T::ice_staticId()))
-	{
-	    d = new T;
-	    d->__copyFrom(b);
-	}
-    }
-    return d;
-}
-
-template<typename P> P 
-checkedCastImpl(const ::Ice::ObjectPrx& b, const ::Ice::Context& ctx)
-{
-    P d = 0;
-    if(b.get())
-    {
-	typedef typename P::element_type T;
-
-	d = dynamic_cast<T*>(b.get());
-	if(!d && b->ice_isA(T::ice_staticId(), ctx))
-	{
-	    d = new T;
-	    d->__copyFrom(b);
-	}
+        d = dynamic_cast<T*>(b.get());
+        if(!d && (context == 0 ? 
+                  b->ice_isA(T::ice_staticId()) :
+                  b->ice_isA(T::ice_staticId(), *context)))
+        {
+            d = new T;
+            d->__copyFrom(b);
+        }
     }
     return d;
 }
@@ -394,14 +450,14 @@ uncheckedCastImpl(const ::Ice::ObjectPrx& b)
     P d = 0;
     if(b)
     {
-	typedef typename P::element_type T;
+        typedef typename P::element_type T;
 
-	d = dynamic_cast<T*>(b.get());
-	if(!d)
-	{
-	    d = new T;
-	    d->__copyFrom(b);
-	}
+        d = dynamic_cast<T*>(b.get());
+        if(!d)
+        {
+            d = new T;
+            d->__copyFrom(b);
+        }
     }
     return d;
 }
@@ -413,9 +469,8 @@ uncheckedCastImpl(const ::Ice::ObjectPrx& b)
 //
 // Helper with type ID.
 //
-ICE_API ::Ice::ObjectPrx checkedCastImpl(const ::Ice::ObjectPrx&, const std::string&, const std::string&);
 ICE_API ::Ice::ObjectPrx checkedCastImpl(const ::Ice::ObjectPrx&, const std::string&, const std::string&,
-                                         const ::Ice::Context&);
+                                         const ::Ice::Context*);
 
 //
 // Specializations for P = ::Ice::ObjectPrx
@@ -423,15 +478,9 @@ ICE_API ::Ice::ObjectPrx checkedCastImpl(const ::Ice::ObjectPrx&, const std::str
 //
 
 template<> inline ::Ice::ObjectPrx 
-checkedCastImpl< ::Ice::ObjectPrx>(const ::Ice::ObjectPrx& b, const std::string& f)
+checkedCastImpl< ::Ice::ObjectPrx>(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context* context)
 {
-    return checkedCastImpl(b, f, "::Ice::Object");
-}
-
-template<> inline ::Ice::ObjectPrx 
-checkedCastImpl< ::Ice::ObjectPrx>(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context& ctx)
-{
-    return checkedCastImpl(b, f, "::Ice::Object", ctx);
+    return checkedCastImpl(b, f, "::Ice::Object", context);
 }
 
 template<> inline ::Ice::ObjectPrx 
@@ -440,39 +489,23 @@ uncheckedCastImpl< ::Ice::ObjectPrx>(const ::Ice::ObjectPrx& b, const std::strin
     ::Ice::ObjectPrx d = 0;
     if(b)
     {
-	d = b->ice_facet(f);
+        d = b->ice_facet(f);
     }
     return d;
 }
 
 template<typename P> P 
-checkedCastImpl(const ::Ice::ObjectPrx& b, const std::string& f)
+checkedCastImpl(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context* context)
 {
     P d = 0;
 
     typedef typename P::element_type T;
-    ::Ice::ObjectPrx bb = checkedCastImpl(b, f, T::ice_staticId());
+    ::Ice::ObjectPrx bb = checkedCastImpl(b, f, T::ice_staticId(), context);
 
     if(bb)
     {
-	d = new T;
-	d->__copyFrom(bb);
-    }
-    return d;
-}
-
-template<typename P> P 
-checkedCastImpl(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context& ctx)
-{
-    P d = 0;
-
-    typedef typename P::element_type T;
-    ::Ice::ObjectPrx bb = checkedCastImpl(b, f, T::ice_staticId(), ctx);
-
-    if(bb)
-    {
-	d = new T;
-	d->__copyFrom(bb);
+        d = new T;
+        d->__copyFrom(bb);
     }
     return d;
 }
@@ -483,11 +516,11 @@ uncheckedCastImpl(const ::Ice::ObjectPrx& b, const std::string& f)
     P d = 0;
     if(b)
     {
-	typedef typename P::element_type T;
+        typedef typename P::element_type T;
 
-	::Ice::ObjectPrx bb = b->ice_facet(f);
-	d = new T;
-	d->__copyFrom(bb);
+        ::Ice::ObjectPrx bb = b->ice_facet(f);
+        d = new T;
+        d->__copyFrom(bb);
     }
     return d;
 }
@@ -501,17 +534,7 @@ template<typename P, typename Y> inline P
 checkedCast(const ::IceInternal::ProxyHandle<Y>& b)
 {
     Y* tag = 0;
-#if defined(_MSC_VER) && (_MSC_VER < 1300)
-    return ::IceInternal::checkedCastHelper<P::element_type>(b, tag);
-#else
-    return ::IceInternal::checkedCastHelper<typename P::element_type>(b, tag);
-#endif
-}
-
-template<typename P, typename Y> inline P 
-checkedCast(const ::IceInternal::ProxyHandle<Y>& b, const ::Ice::Context& ctx)
-{
-    Y* tag = 0;
+    Ice::Context* ctx = 0;
 #if defined(_MSC_VER) && (_MSC_VER < 1300)
     return ::IceInternal::checkedCastHelper<P::element_type>(b, tag, ctx);
 #else
@@ -519,6 +542,17 @@ checkedCast(const ::IceInternal::ProxyHandle<Y>& b, const ::Ice::Context& ctx)
 #endif
 }
 
+template<typename P, typename Y> inline P 
+checkedCast(const ::IceInternal::ProxyHandle<Y>& b, const ::Ice::Context& context)
+{
+    Y* tag = 0;
+#if defined(_MSC_VER) && (_MSC_VER < 1300)
+    return ::IceInternal::checkedCastHelper<P::element_type>(b, tag, &context);
+#else
+    return ::IceInternal::checkedCastHelper<typename P::element_type>(b, tag, &context);
+#endif
+}
+
 template<typename P, typename Y> inline P
 uncheckedCast(const ::IceInternal::ProxyHandle<Y>& b)
 {
@@ -533,19 +567,20 @@ uncheckedCast(const ::IceInternal::ProxyHandle<Y>& b)
 template<typename P> inline P 
 checkedCast(const ::Ice::ObjectPrx& b, const std::string& f)
 {
-    return ::IceInternal::checkedCastImpl<P>(b, f);
+    Ice::Context* ctx = 0;
+    return ::IceInternal::checkedCastImpl<P>(b, f, ctx);
 }
 
 template<typename P> inline P 
-checkedCast(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context& ctx)
+checkedCast(const ::Ice::ObjectPrx& b, const std::string& f, const ::Ice::Context& context)
 {
-    return ::IceInternal::checkedCastImpl<P>(b, f, ctx);
+    return ::IceInternal::checkedCastImpl<P>(b, f, &context);
 }
 
 template<typename P> inline P 
 uncheckedCast(const ::Ice::ObjectPrx& b, const std::string& f)
 {
-    return ::IceInternal::checkedCastImpl<P>(b, f);
+    return ::IceInternal::uncheckedCastImpl<P>(b, f);
 }
 
 #endif
