@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,11 +13,11 @@
 #include <IceBox/IceBox.h>
 #include <IceStorm/IceStorm.h>
 
-#ifndef ICE_STORM_SERVICE_API
-#   ifdef ICE_STORM_SERVICE_API_EXPORTS
-#       define ICE_STORM_SERVICE_API ICE_DECLSPEC_EXPORT
+#ifndef ICE_STORM_API
+#   ifdef ICE_STORM_API_EXPORTS
+#       define ICE_STORM_API ICE_DECLSPEC_EXPORT
 #   else
-#       define ICE_STORM_SERVICE_API ICE_DECLSPEC_IMPORT
+#       define ICE_STORM_API ICE_DECLSPEC_IMPORT
 #   endif
 #endif
 
@@ -27,16 +27,16 @@ namespace IceStorm
 class Service;
 typedef ::IceInternal::Handle< ::IceStorm::Service> ServicePtr;
 
-class ICE_STORM_SERVICE_API Service : public ::IceBox::Service
+class ICE_STORM_API Service : public ::IceBox::Service
 {
 public:
 
     static ServicePtr create(const Ice::CommunicatorPtr&,
-			     const Ice::ObjectAdapterPtr&,
-			     const Ice::ObjectAdapterPtr&,
-			     const std::string&,
-			     const Ice::Identity&,
-			     const std::string&);
+                             const Ice::ObjectAdapterPtr&,
+                             const Ice::ObjectAdapterPtr&,
+                             const std::string&,
+                             const Ice::Identity&,
+                             const std::string&);
 
     virtual void start(const std::string&, const Ice::CommunicatorPtr&, const Ice::StringSeq&) = 0;
 
