@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -20,7 +20,8 @@ class FailureException : public IceUtil::Exception
 public:
     
     FailureException(const char*, int, const std::string&);
-    virtual const std::string ice_name() const;
+    virtual ~FailureException() throw();
+    virtual std::string ice_name() const;
     virtual void ice_print(std::ostream&) const;
     virtual IceUtil::Exception* ice_clone() const;
     virtual void ice_throw() const;
