@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -27,8 +27,9 @@ class AnalyzeTransformVisitor : public ParserVisitor
 {
 public:
 
-    AnalyzeTransformVisitor(XMLOutput&, const UnitPtr&, const TypePtr&, const TypePtr&, const TypePtr&, const TypePtr&,
-                            bool, vector<string>&, vector<string>&);
+    AnalyzeTransformVisitor(XMLOutput&, const UnitPtr&, bool, vector<string>&, vector<string>&);
+
+    void addDatabase(const string&, const TypePtr&, const TypePtr&, const TypePtr&, const TypePtr&);
 
     virtual bool visitClassDefStart(const ClassDefPtr&);
     virtual bool visitStructStart(const StructPtr&);
@@ -81,42 +82,58 @@ private:
 ////////////////////////////////////
 
 FreezeScript::AnalyzeTransformVisitor::AnalyzeTransformVisitor(XMLOutput& out, const UnitPtr& newUnit,
-                                                               const TypePtr& oldKey, const TypePtr& newKey,
-                                                               const TypePtr& oldValue, const TypePtr& newValue,
                                                                bool ignoreTypeChanges, vector<string>& missingTypes,
                                                                vector<string>& errors) :
     _out(out), _newUnit(newUnit), _ignoreTypeChanges(ignoreTypeChanges), _missingTypes(missingTypes), _errors(errors)
 {
-    out << se("database");
+}
 
-    string oldKeyName = typeToString(oldKey);
-    string newKeyName = typeToString(newKey);
+void
+FreezeScript::AnalyzeTransformVisitor::addDatabase(const string& name, const TypePtr& oldKey, const TypePtr& newKey,
+                                                   const TypePtr& oldValue, const TypePtr& newValue)
+{
+    _out << "\n";
+    _out << se("database");
+
+    if(!name.empty())
+    {
+        _out << attr("name", name);
+    }
+
+    string oldKeyName = oldKey ? typeToString(oldKey) : string("UNKNOWN");
+    string newKeyName = newKey ? typeToString(newKey) : string("UNKNOWN");
     if(oldKeyName == newKeyName)
     {
-        out << attr("key", oldKeyName);
+        _out << attr("key", oldKeyName);
     }
     else
     {
-        out << attr("key", oldKeyName + "," + newKeyName);
+        _out << attr("key", oldKeyName + "," + newKeyName);
     }
 
-    string oldValueName = typeToString(oldValue);
-    string newValueName = typeToString(newValue);
+    string oldValueName = oldValue ? typeToString(oldValue) : string("UNKNOWN");
+    string newValueName = newValue ? typeToString(newValue) : string("UNKNOWN");
     if(oldValueName == newValueName)
     {
-        out << attr("value", oldValueName);
+        _out << attr("value", oldValueName);
     }
     else
     {
-        out << attr("value", oldValueName + "," + newValueName);
+        _out << attr("value", oldValueName + "," + newValueName);
     }
 
-    out << se("record");
-    compareTypes("database key", oldKey, newKey);
-    compareTypes("database value", oldValue, newValue);
-    out << ee;
+    _out << se("record");
+    if(oldKey && newKey)
+    {
+        compareTypes("database key", oldKey, newKey);
+    }
+    if(oldValue && newValue)
+    {
+        compareTypes("database value", oldValue, newValue);
+    }
+    _out << ee;
 
-    out << ee;
+    _out << ee;
 }
 
 bool
@@ -157,8 +174,8 @@ FreezeScript::AnalyzeTransformVisitor::visitClassDefStart(const ClassDefPtr& v)
         return false;
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- class " << scoped << " -->";
     _out << se("transform") << attr("type", scoped);
 
@@ -202,8 +219,8 @@ FreezeScript::AnalyzeTransformVisitor::visitStructStart(const StructPtr& v)
         return false;
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- struct " << scoped << " -->";
     _out << se("transform") << attr("type", scoped);
 
@@ -247,8 +264,8 @@ FreezeScript::AnalyzeTransformVisitor::visitSequence(const SequencePtr& v)
         return;
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- sequence " << scoped << " -->";
     _out << se("transform") << attr("type", scoped);
 
@@ -288,8 +305,8 @@ FreezeScript::AnalyzeTransformVisitor::visitDictionary(const DictionaryPtr& v)
         return;
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- dictionary " << scoped << " -->";
     _out << se("transform") << attr("type", scoped);
 
@@ -340,8 +357,8 @@ FreezeScript::AnalyzeTransformVisitor::visitEnum(const EnumPtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- enum " << scoped << " -->";
     _out << se("transform") << attr("type", scoped);
 
@@ -352,12 +369,12 @@ FreezeScript::AnalyzeTransformVisitor::visitEnum(const EnumPtr& v)
         map<string, int>::const_iterator q = m.find((*p)->name());
         if(q == m.end())
         {
-            _out.nl();
+            _out.newline();
             _out << "<!-- NOTICE: enumerator `" << (*p)->name() << "' has been removed -->";
         }
         else if(q->second != i)
         {
-            _out.nl();
+            _out.newline();
             _out << "<!-- NOTICE: enumerator `" << (*p)->name() << "' has changed position -->";
         }
     }
@@ -389,7 +406,7 @@ FreezeScript::AnalyzeTransformVisitor::compareMembers(const DataMemberList& oldM
         q = newMap.find(name);
         if(q == newMap.end())
         {
-            _out.nl();
+            _out.newline();
             _out << "<!-- NOTICE: " << name << " has been removed -->";
         }
         else
@@ -412,7 +429,7 @@ FreezeScript::AnalyzeTransformVisitor::compareMembers(const DataMemberList& oldM
     //
     for(q = newMap.begin(); q != newMap.end(); ++q)
     {
-        _out.nl();
+        _out.newline();
         _out << "<!-- NOTICE: " << q->first << " has been added -->";
     }
 }
@@ -816,7 +833,7 @@ FreezeScript::AnalyzeTransformVisitor::typeChange(const string& desc, const Type
 
     if(_ignoreTypeChanges)
     {
-        _out.nl();
+        _out.newline();
         _out << "<!-- NOTICE: " << desc << " has changed from ";
         if(b1)
         {
@@ -954,8 +971,8 @@ FreezeScript::AnalyzeInitVisitor::visitClassDefStart(const ClassDefPtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- class " << scoped << " -->";
     _out << se("init") << attr("type", scoped);
     _out << ee;
@@ -986,8 +1003,8 @@ FreezeScript::AnalyzeInitVisitor::visitStructStart(const StructPtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- struct " << scoped << " -->";
     _out << se("init") << attr("type", scoped);
     _out << ee;
@@ -1018,8 +1035,8 @@ FreezeScript::AnalyzeInitVisitor::visitSequence(const SequencePtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- sequence " << scoped << " -->";
     _out << se("init") << attr("type", scoped);
     _out << ee;
@@ -1048,8 +1065,8 @@ FreezeScript::AnalyzeInitVisitor::visitDictionary(const DictionaryPtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- dictionary " << scoped << " -->";
     _out << se("init") << attr("type", scoped);
     _out << ee;
@@ -1078,8 +1095,8 @@ FreezeScript::AnalyzeInitVisitor::visitEnum(const EnumPtr& v)
         }
     }
 
-    _out.nl();
-    _out.nl();
+    _out.newline();
+    _out.newline();
     _out << "<!-- enum " << scoped << " -->";
     _out << se("init") << attr("type", scoped);
     _out << ee;
@@ -1093,7 +1110,7 @@ FreezeScript::AnalyzeInitVisitor::typeChange(const TypePtr& t, const string& sco
     ContainedPtr c = ContainedPtr::dynamicCast(t);
     ProxyPtr p = ProxyPtr::dynamicCast(t);
 
-    _out.nl();
+    _out.newline();
     _out << "<!-- NOTICE: " << scoped << " has changed from ";
     if(b)
     {
@@ -1112,27 +1129,34 @@ FreezeScript::AnalyzeInitVisitor::typeChange(const TypePtr& t, const string& sco
 }
 
 FreezeScript::TransformAnalyzer::TransformAnalyzer(const UnitPtr& oldUnit, const UnitPtr& newUnit,
-                                                   bool ignoreTypeChanges) :
-    _old(oldUnit), _new(newUnit), _ignoreTypeChanges(ignoreTypeChanges)
+                                                   bool ignoreTypeChanges, ostream& os, vector<string>& missingTypes,
+                                                   vector<string>& errors) :
+    _old(oldUnit), _new(newUnit), _out(os),
+    _visitor(new AnalyzeTransformVisitor(_out, newUnit, ignoreTypeChanges, missingTypes, errors))
 {
+    _out << se("transformdb");
+}
+
+FreezeScript::TransformAnalyzer::~TransformAnalyzer()
+{
+    delete _visitor;
 }
 
 void
-FreezeScript::TransformAnalyzer::analyze(const TypePtr& oldKey, const TypePtr& newKey, const TypePtr& oldValue,
-                                         const TypePtr& newValue, ostream& os, vector<string>& missingTypes,
-                                         vector<string>& errors)
+FreezeScript::TransformAnalyzer::addDatabase(const string& name, const TypePtr& oldKey, const TypePtr& newKey,
+                                             const TypePtr& oldValue, const TypePtr& newValue)
 {
-    XMLOutput out(os);
+    _visitor->addDatabase(name, oldKey, newKey, oldValue, newValue);
+}
 
-    out << se("transformdb");
+void
+FreezeScript::TransformAnalyzer::finish()
+{
+    _old->visit(_visitor, false);
 
-    AnalyzeTransformVisitor transformVisitor(out, _new, oldKey, newKey, oldValue, newValue, _ignoreTypeChanges,
-                                             missingTypes, errors);
-    _old->visit(&transformVisitor, false);
-
-    AnalyzeInitVisitor initVisitor(out, _old);
+    AnalyzeInitVisitor initVisitor(_out, _old);
     _new->visit(&initVisitor, false);
 
-    out << ee;
-    out << '\n';
+    _out << ee;
+    _out << '\n';
 }
