@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,24 +11,32 @@
 #define FREEZE_SCRIPT_TRANSFORM_ANALYZER_H
 
 #include <Slice/Parser.h>
+#include <IceUtil/OutputUtil.h>
 #include <ostream>
 
 namespace FreezeScript
 {
 
+class AnalyzeTransformVisitor;
+
 class TransformAnalyzer
 {
 public:
 
-    TransformAnalyzer(const Slice::UnitPtr&, const Slice::UnitPtr&, bool);
-    void analyze(const Slice::TypePtr&, const Slice::TypePtr&, const Slice::TypePtr&, const Slice::TypePtr&,
-                 std::ostream&, std::vector<std::string>&, std::vector<std::string>&);
+    TransformAnalyzer(const Slice::UnitPtr&, const Slice::UnitPtr&, bool, std::ostream&, std::vector<std::string>&,
+                      std::vector<std::string>&);
+    ~TransformAnalyzer();
+
+    void addDatabase(const std::string&, const Slice::TypePtr&, const Slice::TypePtr&, const Slice::TypePtr&,
+                     const Slice::TypePtr&);
+    void finish();
 
 private:
 
     Slice::UnitPtr _old;
     Slice::UnitPtr _new;
-    bool _ignoreTypeChanges;
+    IceUtil::XMLOutput _out;
+    AnalyzeTransformVisitor* _visitor;
 };
 
 }
