@@ -1,12 +1,13 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+#include <IceUtil/DisableWarnings.h>
 #include <Ice/CommunicatorI.h>
 #include <Ice/Instance.h>
 #include <Ice/Properties.h>
@@ -34,7 +35,7 @@ IceUtil::Handle<IceInternal::GC> theCollector = 0;
 struct GarbageCollectorStats
 {
     GarbageCollectorStats() :
-	runs(0), examined(0), collected(0)
+        runs(0), examined(0), collected(0)
     {
     }
     int runs;
@@ -48,7 +49,6 @@ static IceUtil::StaticMutex gcMutex = ICE_STATIC_MUTEX_INITIALIZER;
 static GarbageCollectorStats gcStats;
 static int gcTraceLevel;
 static string gcTraceCat;
-static LoggerPtr gcLogger;
 static int gcInterval;
 
 static void
@@ -56,15 +56,15 @@ printGCStats(const IceInternal::GCStats& stats)
 {
     if(gcTraceLevel)
     {
-	if(gcTraceLevel > 1)
-	{
-	    Trace out(gcLogger, gcTraceCat);
-	    out << stats.collected << "/" << stats.examined << ", " << stats.time * 1000 << "ms";
-	}
-	++gcStats.runs;
-	gcStats.examined += stats.examined;
-	gcStats.collected += stats.collected;
-	gcStats.time += stats.time;
+        if(gcTraceLevel > 1)
+        {
+            Trace out(getProcessLogger(), gcTraceCat);
+            out << stats.collected << "/" << stats.examined << ", " << stats.time * 1000 << "ms";
+        }
+        ++gcStats.runs;
+        gcStats.examined += stats.examined;
+        gcStats.collected += stats.collected;
+        gcStats.time += stats.time;
     }
 }
 
@@ -73,37 +73,37 @@ Ice::CommunicatorI::destroy()
 {
     if(_instance->destroy())
     {
-	IceUtil::StaticMutex::Lock sync(gcMutex);
+        IceUtil::StaticMutex::Lock sync(gcMutex);
 
-	//
-	// Wait for the collector thread to stop if this is the last communicator
-	// to be destroyed.
-	//
-	bool last = (--communicatorCount == 0);
-	if(last && gcInterval > 0 && theCollector)
-	{
-	    theCollector->stop();
-	}
-
-	if(theCollector)
+        //
+        // Wait for the collector thread to stop if this is the last communicator
+        // to be destroyed.
+        //
+        bool last = (--communicatorCount == 0);
+        if(last && gcInterval > 0 && theCollector)
         {
-	    theCollector->collectGarbage(); // Collect whenever a communicator is destroyed.
-    	}
+            theCollector->stop();
+        }
 
-	if(last)
-	{
-	    if(gcTraceLevel)
-	    {
-		Trace out(gcLogger, gcTraceCat);
-		out << "totals: " << gcStats.collected << "/" << gcStats.examined << ", "
-		    << gcStats.time * 1000 << "ms" << ", " << gcStats.runs << " run";
-		if(gcStats.runs != 1)
-		{
-		    out << "s";
-		}
-	    }
-	    theCollector = 0; // Force destruction of the collector.
-	}
+        if(theCollector)
+        {
+            theCollector->collectGarbage(); // Collect whenever a communicator is destroyed.
+        }
+
+        if(last)
+        {
+            if(gcTraceLevel)
+            {
+                Trace out(getProcessLogger(), gcTraceCat);
+                out << "totals: " << gcStats.collected << "/" << gcStats.examined << ", "
+                    << gcStats.time * 1000 << "ms" << ", " << gcStats.runs << " run";
+                if(gcStats.runs != 1)
+                {
+                    out << "s";
+                }
+            }
+            theCollector = 0; // Force destruction of the collector.
+        }
     }
 }
 
@@ -119,6 +119,12 @@ Ice::CommunicatorI::waitForShutdown()
     _instance->objectAdapterFactory()->waitForShutdown();
 }
 
+bool
+Ice::CommunicatorI::isShutdown() const
+{
+    return _instance->objectAdapterFactory()->isShutdown();
+}
+
 ObjectPrx
 Ice::CommunicatorI::stringToProxy(const string& s) const
 {
@@ -131,6 +137,12 @@ Ice::CommunicatorI::proxyToString(const ObjectPrx& proxy) const
     return _instance->proxyFactory()->proxyToString(proxy);
 }
 
+ObjectPrx
+Ice::CommunicatorI::propertyToProxy(const string& p) const
+{
+    return _instance->proxyFactory()->propertyToProxy(p);
+}
+
 Identity
 Ice::CommunicatorI::stringToIdentity(const string& s) const
 {
@@ -146,7 +158,7 @@ Ice::CommunicatorI::identityToString(const Identity& ident) const
 ObjectAdapterPtr
 Ice::CommunicatorI::createObjectAdapter(const string& name)
 {
-    return createObjectAdapterWithEndpoints(name, getProperties()->getProperty(name + ".Endpoints"));
+    return _instance->objectAdapterFactory()->createObjectAdapter(name, "", 0);
 }
 
 ObjectAdapterPtr
@@ -218,24 +230,21 @@ Ice::CommunicatorI::setDefaultLocator(const LocatorPrx& locator)
 Ice::Context
 Ice::CommunicatorI::getDefaultContext() const
 {
-    return _instance->getDefaultContext();
+    return _instance->getDefaultContext()->getValue();
 }
 
 void
-Ice::Communicator::setDefaultContext(const Context& ctx)
-{
-    //
-    // We know there is only one Communicator implementation!
-    //
-    dynamic_cast<Ice::CommunicatorI*>(this)->setDefaultContextI(ctx);
-}
-
-void
-Ice::CommunicatorI::setDefaultContextI(const Context& ctx)
+Ice::CommunicatorI::setDefaultContext(const Context& ctx)
 {
     _instance->setDefaultContext(ctx);
 }
 
+Ice::ImplicitContextPtr
+Ice::CommunicatorI::getImplicitContext() const
+{
+    return _instance->getImplicitContext();
+}
+
 
 PluginManagerPtr
 Ice::CommunicatorI::getPluginManager() const
@@ -254,48 +263,47 @@ Ice::CommunicatorI::CommunicatorI(const InitializationData& initData)
     __setNoDelete(true);
     try
     {
-	const_cast<InstancePtr&>(_instance) = new Instance(this, initData);
+        const_cast<InstancePtr&>(_instance) = new Instance(this, initData);
 
         //
         // Keep a reference to the dynamic library list to ensure
         // the libraries are not unloaded until this Communicator's
         // destructor is invoked.
         //
-	const_cast<DynamicLibraryListPtr&>(_dynamicLibraryList) = _instance->dynamicLibraryList();
+        const_cast<DynamicLibraryListPtr&>(_dynamicLibraryList) = _instance->dynamicLibraryList();
     }
     catch(...)
     {
-	__setNoDelete(false);
-	throw;
+        __setNoDelete(false);
+        throw;
     }
     __setNoDelete(false);
 
     {
-	//
-	// If this is the first communicator that is created, use that communicator's
-	// property settings to determine whether to start the garbage collector.
-	// We remember that communicator's trace and logger settings so the garbage
-	// collector can continue to log messages even if the first communicator that
-	// is created isn't the last communicator to be destroyed.
-	//
-	IceUtil::StaticMutex::Lock sync(gcMutex);
-	static bool gcOnce = true;
-	if(gcOnce)
-	{
-	    gcTraceLevel = _instance->traceLevels()->gc;
-	    gcTraceCat = _instance->traceLevels()->gcCat;
-	    gcLogger = _instance->initializationData().logger;
-	    gcInterval = _instance->initializationData().properties->getPropertyAsInt("Ice.GC.Interval");
-	    gcOnce = false;
-	}
-	if(++communicatorCount == 1)
-	{
-	    theCollector = new IceInternal::GC(gcInterval, printGCStats);
-	    if(gcInterval > 0)
-	    {
-		theCollector->start();
-	    }
-	}
+        //
+        // If this is the first communicator that is created, use that communicator's
+        // property settings to determine whether to start the garbage collector.
+        // We remember that communicator's trace and logger settings so the garbage
+        // collector can continue to log messages even if the first communicator that
+        // is created isn't the last communicator to be destroyed.
+        //
+        IceUtil::StaticMutex::Lock sync(gcMutex);
+        static bool gcOnce = true;
+        if(gcOnce)
+        {
+            gcTraceLevel = _instance->traceLevels()->gc;
+            gcTraceCat = _instance->traceLevels()->gcCat;
+            gcInterval = _instance->initializationData().properties->getPropertyAsInt("Ice.GC.Interval");
+            gcOnce = false;
+        }
+        if(++communicatorCount == 1)
+        {
+            theCollector = new IceInternal::GC(gcInterval, printGCStats);
+            if(gcInterval > 0)
+            {
+                theCollector->start();
+            }
+        }
     }
 }
 
@@ -303,8 +311,8 @@ Ice::CommunicatorI::~CommunicatorI()
 {
     if(!_instance->destroyed())
     {
-	Warning out(_instance->initializationData().logger);
-	out << "Ice::Communicator::destroy() has not been called";
+        Warning out(_instance->initializationData().logger);
+        out << "Ice::Communicator::destroy() has not been called";
     }
 }
 
@@ -313,11 +321,11 @@ Ice::CommunicatorI::finishSetup(int& argc, char* argv[])
 {
     try
     {
-	_instance->finishSetup(argc, argv);
+        _instance->finishSetup(argc, argv);
     }
     catch(...)
     {
-	_instance->destroy();
-	throw;
+        _instance->destroy();
+        throw;
     }
 }
