@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -8,6 +8,8 @@
 // **********************************************************************
 
 #include <Ice/EventLoggerI.h>
+#include <Ice/ImplicitContextI.h>
+#include <Ice/Service.h>
 
 extern "C" BOOL WINAPI _CRT_INIT(HINSTANCE, DWORD, LPVOID);
 
@@ -15,7 +17,7 @@ extern "C"
 {
 
 BOOL WINAPI
-#ifdef _BCPLUSPLUS__
+#ifdef __BCPLUSPLUS__
 DllMain(HINSTANCE hDLL, DWORD reason, LPVOID reserved)
 {
 #else
@@ -26,10 +28,14 @@ ice_DLL_Main(HINSTANCE hDLL, DWORD reason, LPVOID reserved)
         return FALSE;
     }
 #endif
-
     if(reason == DLL_PROCESS_ATTACH)
     {
         Ice::EventLoggerI::setModuleHandle(hDLL);
+        Ice::Service::setModuleHandle(hDLL);
+    }
+    else if(reason == DLL_THREAD_DETACH)
+    {
+        Ice::ImplicitContextI::cleanupThread();
     }
 
     return TRUE;
