@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,6 +13,7 @@
 #include <IceUtil/Shared.h>
 #include <Ice/DefaultsAndOverridesF.h>
 #include <Ice/PropertiesF.h>
+#include <Ice/Endpoint.h>
 
 namespace IceInternal
 {
@@ -25,10 +26,10 @@ public:
 
     std::string defaultHost;
     std::string defaultProtocol;
-    std::string defaultRouter;
-    std::string defaultLocator;
     bool defaultCollocationOptimization;
+    Ice::EndpointSelectionType defaultEndpointSelection;
     int defaultLocatorCacheTimeout;
+    bool defaultPreferSecure;
 
     bool overrideTimeout;
     Ice::Int overrideTimeoutValue;
@@ -36,6 +37,8 @@ public:
     Ice::Int overrideConnectTimeoutValue;
     bool overrideCompress;
     bool overrideCompressValue;
+    bool overrideSecure;
+    bool overrideSecureValue;
 };
 
 }
