@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -66,10 +66,20 @@ IceInternal::DynamicLibrary::loadEntryPoint(const string& entryPoint, bool useIc
         libName = libSpec;
         if(useIceVersion)
         {
-	    int majorVersion = (ICE_INT_VERSION / 10000);
-	    int minorVersion = (ICE_INT_VERSION / 100) - majorVersion * 100;
-	    ostringstream os;
-	    os << majorVersion * 10 + minorVersion;
+            int majorVersion = (ICE_INT_VERSION / 10000);
+            int minorVersion = (ICE_INT_VERSION / 100) - majorVersion * 100;
+            ostringstream os;
+            os << majorVersion * 10 + minorVersion;
+            
+            int patchVersion = ICE_INT_VERSION % 100;
+            if(patchVersion > 50)
+            {
+                os << 'b';
+                if(patchVersion >= 52)
+                {
+                    os << (patchVersion - 50);
+                }
+            }
             version = os.str();
         }
     }
@@ -91,7 +101,7 @@ IceInternal::DynamicLibrary::loadEntryPoint(const string& entryPoint, bool useIc
     lib = "lib" + libName;
     if(!version.empty()) 
     {
-	lib += "." + version;
+        lib += "." + version;
     }
     lib += ".dylib";
 #elif defined(__hpux)
@@ -102,7 +112,7 @@ IceInternal::DynamicLibrary::loadEntryPoint(const string& entryPoint, bool useIc
     }
     else
     {
-	lib += ".sl";
+        lib += ".sl";
     }
 #elif defined(_AIX)
     lib = "lib" + libName + ".a(lib" + libName + ".so";
