@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -35,6 +35,7 @@ public:
     virtual std::string type() const;
     virtual std::string toString() const;
     virtual void initialize(int);
+    virtual void checkSendSize(const Buffer&, size_t);
 
 private:
 
@@ -53,7 +54,7 @@ private:
 
     const std::string _desc;
 #ifdef _WIN32
-    const bool _isPeerLocal;
+    int _maxPacketSize;
 #endif
 };
 
