@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -42,6 +42,8 @@ Client::usage()
         "-v, --version        Display the Ice version.\n"
         "\n"
         "Commands:\n"
+        "start SERVICE        Start a service.\n"
+        "stop SERVICE         Stop a service.\n"
         "shutdown             Shutdown the server.\n"
         ;
 }
@@ -56,24 +58,24 @@ Client::run(int argc, char* argv[])
     vector<string> commands;
     try
     {
-    	commands = opts.parse(argc, (const char**)argv);
+        commands = opts.parse(argc, (const char**)argv);
     }
-    catch(const IceUtil::Options::BadOpt& e)
+    catch(const IceUtil::BadOptException& e)
     {
         cerr << e.reason << endl;
-	usage();
-	return EXIT_FAILURE;
+        usage();
+        return EXIT_FAILURE;
     }
 
-    if(opts.isSet("h") || opts.isSet("help"))
+    if(opts.isSet("help"))
     {
-	usage();
-	return EXIT_SUCCESS;
+        usage();
+        return EXIT_SUCCESS;
     }
-    if(opts.isSet("v") || opts.isSet("version"))
+    if(opts.isSet("version"))
     {
-	cout << ICE_STRING_VERSION << endl;
-	return EXIT_SUCCESS;
+        cout << ICE_STRING_VERSION << endl;
+        return EXIT_SUCCESS;
     }
 
     if(commands.empty())
@@ -84,40 +86,31 @@ Client::run(int argc, char* argv[])
 
     PropertiesPtr properties = communicator()->getProperties();
     Identity managerIdentity;
-    string identity = properties->getProperty("IceBox.ServiceManager.Identity");
-    if(!identity.empty())
-    {
-        managerIdentity = communicator()->stringToIdentity(identity);
-    }
-    else
-    {
-        managerIdentity.category = properties->getPropertyWithDefault("IceBox.InstanceName", "IceBox");
-	managerIdentity.name = "ServiceManager";
-    }
+    managerIdentity.category = properties->getPropertyWithDefault("IceBox.InstanceName", "IceBox");
+    managerIdentity.name = "ServiceManager";
 
     string managerProxy;
-
     if(properties->getProperty("Ice.Default.Locator").empty())
     {
-	string managerEndpoints = properties->getProperty("IceBox.ServiceManager.Endpoints");
-	if(managerEndpoints.empty())
-	{
-	    cerr << appName() << ": property `IceBox.ServiceManager.Endpoints' is not set" << endl;
-	    return EXIT_FAILURE;
-	}
+        string managerEndpoints = properties->getProperty("IceBox.ServiceManager.Endpoints");
+        if(managerEndpoints.empty())
+        {
+            cerr << appName() << ": property `IceBox.ServiceManager.Endpoints' is not set" << endl;
+            return EXIT_FAILURE;
+        }
 
-	managerProxy = "\"" + communicator()->identityToString(managerIdentity) + "\" :" + managerEndpoints;
+        managerProxy = "\"" + communicator()->identityToString(managerIdentity) + "\" :" + managerEndpoints;
     }
     else
     {
-	string managerAdapterId = properties->getProperty("IceBox.ServiceManager.AdapterId");
-	if(managerAdapterId.empty())
-	{
-	    cerr << appName() << ": property `IceBox.ServiceManager.AdapterId' is not set" << endl;
-	    return EXIT_FAILURE;
-	}
+        string managerAdapterId = properties->getProperty("IceBox.ServiceManager.AdapterId");
+        if(managerAdapterId.empty())
+        {
+            cerr << appName() << ": property `IceBox.ServiceManager.AdapterId' is not set" << endl;
+            return EXIT_FAILURE;
+        }
 
-	managerProxy = "\"" + communicator()->identityToString(managerIdentity) + "\" @" + managerAdapterId;
+        managerProxy = "\"" + communicator()->identityToString(managerIdentity) + "\" @" + managerAdapterId;
     }
 
     ObjectPrx base = communicator()->stringToProxy(managerProxy);
@@ -143,6 +136,7 @@ Client::run(int argc, char* argv[])
         }
     }
 
+    
     vector<string>::const_iterator r;
     for(r = commands.begin(); r != commands.end(); ++r)
     {
@@ -150,6 +144,48 @@ Client::run(int argc, char* argv[])
         {
             manager->shutdown();
         }
+        else if((*r) == "start")
+        {
+            if(++r == commands.end())
+            {
+                cerr << appName() << ": no service name specified." << endl;
+                return EXIT_FAILURE;
+            }
+
+            try
+            {
+                manager->startService(*r);
+            }
+            catch(const IceBox::NoSuchServiceException&)
+            {
+                cerr << appName() << ": unknown service `" << *r << "'" << endl;
+            }
+            catch(const IceBox::AlreadyStartedException&)
+            {
+                cerr << appName() << ": service already started." << endl;
+            }
+        }
+        else if((*r) == "stop")
+        {
+            if(++r == commands.end())
+            {
+                cerr << appName() << ": no service name specified." << endl;
+                return EXIT_FAILURE;
+            }
+
+            try
+            {
+                manager->stopService(*r);
+            }
+            catch(const IceBox::NoSuchServiceException&)
+            {
+                cerr << appName() << ": unknown service `" << *r << "'" << endl;
+            }
+            catch(const IceBox::AlreadyStoppedException&)
+            {
+                cerr << appName() << ": service already stopped." << endl;
+            }
+        }
         else
         {
             cerr << appName() << ": unknown command `" << *r << "'" << endl;
