@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,6 +13,8 @@
 #include <IceUtil/OutputUtil.h>
 #include <IceXML/Parser.h>
 #include <IceGrid/Admin.h>
+#include <IceGrid/Internal.h>
+#include <set>
 
 namespace IceGrid
 {
@@ -22,11 +24,11 @@ class Resolver
 {
 public:
 
-    Resolver(const ApplicationDescriptor&, const Ice::CommunicatorPtr&);
+    Resolver(const ApplicationDescriptor&, const Ice::CommunicatorPtr&, bool);
     Resolver(const Resolver&, const std::map<std::string, std::string>&, bool);
-    Resolver(const std::string&, const NodeInfo&, const Ice::CommunicatorPtr&);
+    Resolver(const InternalNodeInfoPtr&, const Ice::CommunicatorPtr&);
 
-    std::string operator()(const std::string&, const std::string& = std::string(), bool = true, bool = true) const;
+    std::string operator()(const std::string&, const std::string& = std::string(), bool = true) const;
     Ice::StringSeq operator()(const Ice::StringSeq&, const std::string&) const;
     DistributionDescriptor operator()(const DistributionDescriptor&) const;
     PropertyDescriptorSeq operator()(const PropertyDescriptorSeq&, const std::string& = std::string("property")) const;
@@ -37,6 +39,7 @@ public:
 
     std::string asInt(const std::string&, const std::string& = std::string()) const;
     std::string asFloat(const std::string&, const std::string& = std::string()) const;
+    std::string asId(const std::string&, const std::string& = std::string(), bool = false) const;
 
     void setReserved(const std::string&, const std::string&);
     void setContext(const std::string&);
@@ -52,10 +55,11 @@ public:
     bool hasReplicaGroup(const std::string&) const;
     
     Ice::CommunicatorPtr getCommunicator() const { return _communicator; }
+    bool warningEnabled() const { return _enableWarning; }
 
 private:
 
-    std::string substitute(const std::string&, bool = false) const;
+    std::string substitute(const std::string&, bool, bool) const;
     std::string getVariable(const std::string&, bool, bool&) const;
     PropertyDescriptorSeq getProperties(const Ice::StringSeq&, std::set<std::string>&) const;
 
@@ -65,6 +69,7 @@ private:
     const ApplicationDescriptor* _application;
     const Ice::CommunicatorPtr _communicator;
     const bool _escape;
+    const bool _enableWarning;
     std::string _context;
     std::map<std::string, std::string> _variables;
     std::map<std::string, std::string> _parameters;
@@ -77,7 +82,7 @@ class CommunicatorHelper
 {
 public:
 
-    CommunicatorHelper(const Ice::CommunicatorPtr&, const CommunicatorDescriptorPtr&);
+    CommunicatorHelper(const CommunicatorDescriptorPtr&);
     CommunicatorHelper() { }
     virtual ~CommunicatorHelper() { }
 
@@ -85,20 +90,19 @@ public:
     bool operator!=(const CommunicatorHelper&) const;
 
     virtual void getIds(std::multiset<std::string>&, std::multiset<Ice::Identity>&) const;
+    virtual void getReplicaGroups(std::set<std::string>&) const;
     
-    void print(IceUtil::Output&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&) const;
 
 protected:
 
     void printDbEnv(IceUtil::Output&, const DbEnvDescriptor&) const;
-    void printObjectAdapter(IceUtil::Output&, const AdapterDescriptor&) const;
+    void printObjectAdapter(const Ice::CommunicatorPtr&, IceUtil::Output&, const AdapterDescriptor&) const;
     void printPropertySet(IceUtil::Output&, const PropertySetDescriptor&) const;
     virtual std::string getProperty(const std::string&) const;
 
     void instantiateImpl(const CommunicatorDescriptorPtr&, const Resolver&) const;
     
-    Ice::CommunicatorPtr _communicator;
-
 private:
 
     CommunicatorDescriptorPtr _desc;
@@ -108,20 +112,22 @@ class ServiceHelper : public CommunicatorHelper
 {
 public:
 
-    ServiceHelper(const Ice::CommunicatorPtr&, const ServiceDescriptorPtr&);
+    ServiceHelper(const ServiceDescriptorPtr&);
     ServiceHelper() { }
 
     bool operator==(const ServiceHelper&) const;
     bool operator!=(const ServiceHelper&) const;    
 
     ServiceDescriptorPtr getDescriptor() const;
-    ServiceDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&) const;
+    ServiceDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&, 
+                                     const PropertySetDescriptorDict&) const;
 
-    void print(IceUtil::Output&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&) const;
 
 protected:
 
-    void instantiateImpl(const ServiceDescriptorPtr&, const Resolver&, const PropertyDescriptorSeq&) const;
+    void instantiateImpl(const ServiceDescriptorPtr&, const Resolver&, const PropertyDescriptorSeq&,
+                         const PropertySetDescriptorDict&) const;
 
 private:
     
@@ -132,21 +138,22 @@ class ServerHelper : public CommunicatorHelper, public IceUtil::SimpleShared
 {
 public:
 
-    ServerHelper(const Ice::CommunicatorPtr&, const ServerDescriptorPtr&);
+    ServerHelper(const ServerDescriptorPtr&);
     ServerHelper() { }
 
     bool operator==(const ServerHelper&) const;
     bool operator!=(const ServerHelper&) const;    
 
     ServerDescriptorPtr getDescriptor() const;
-    virtual ServerDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&) const;
+    virtual ServerDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&,
+                                            const PropertySetDescriptorDict&) const;
 
-    void print(IceUtil::Output&) const;
-    void print(IceUtil::Output&, const std::string&, const std::string&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&, const ServerInfo&) const;
 
 protected:
 
-    void printImpl(IceUtil::Output&, const std::string&, const std::string&) const;
+    void printImpl(const Ice::CommunicatorPtr&, IceUtil::Output&, const ServerInfo&) const;
     void instantiateImpl(const ServerDescriptorPtr&, const Resolver&, const PropertyDescriptorSeq&) const;
 
 private:
@@ -160,10 +167,10 @@ class InstanceHelper
 protected:
 
     std::map<std::string, std::string> instantiateParams(const Resolver&, 
-							 const std::string&, 
-							 const std::map<std::string, std::string>&,
-							 const std::vector<std::string>&,
-							 const std::map<std::string, std::string>&) const;
+                                                         const std::string&, 
+                                                         const std::map<std::string, std::string>&,
+                                                         const std::vector<std::string>&,
+                                                         const std::map<std::string, std::string>&) const;
 };
 
 
@@ -171,15 +178,16 @@ class ServiceInstanceHelper : public InstanceHelper
 {
 public:
 
-    ServiceInstanceHelper(const Ice::CommunicatorPtr&, const ServiceInstanceDescriptor&);
+    ServiceInstanceHelper(const ServiceInstanceDescriptor&);
 
     bool operator==(const ServiceInstanceHelper&) const;
     bool operator!=(const ServiceInstanceHelper&) const;
 
-    ServiceInstanceDescriptor instantiate(const Resolver&) const;
+    ServiceInstanceDescriptor instantiate(const Resolver&, const PropertySetDescriptorDict&) const;
     void getIds(std::multiset<std::string>&, std::multiset<Ice::Identity>&) const;
+    void getReplicaGroups(std::set<std::string>&) const;
 
-    void print(IceUtil::Output&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&) const;
 
 private:
     
@@ -192,22 +200,25 @@ class IceBoxHelper : public ServerHelper
 {
 public:
 
-    IceBoxHelper(const Ice::CommunicatorPtr&, const IceBoxDescriptorPtr&);
+    IceBoxHelper(const IceBoxDescriptorPtr&);
     IceBoxHelper() { }
 
     bool operator==(const IceBoxHelper&) const;
     bool operator!=(const IceBoxHelper&) const;    
 
-    virtual ServerDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&) const;
+    virtual ServerDescriptorPtr instantiate(const Resolver&, const PropertyDescriptorSeq&, 
+                                            const PropertySetDescriptorDict&) const;
 
     virtual void getIds(std::multiset<std::string>&, std::multiset<Ice::Identity>&) const;
+    virtual void getReplicaGroups(std::set<std::string>&) const;
 
-    void print(IceUtil::Output&) const;
-    void print(IceUtil::Output&, const std::string&, const std::string&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&) const;
+    void print(const Ice::CommunicatorPtr&, IceUtil::Output&, const ServerInfo&) const;
 
 protected:
 
-    void instantiateImpl(const IceBoxDescriptorPtr&, const Resolver&, const PropertyDescriptorSeq&) const;
+    void instantiateImpl(const IceBoxDescriptorPtr&, const Resolver&, const PropertyDescriptorSeq&,
+                         const PropertySetDescriptorDict&) const;
 
 private:
     
@@ -235,6 +246,7 @@ public:
     ServerDescriptorPtr getServerInstance() const;
 
     void getIds(std::multiset<std::string>&, std::multiset<Ice::Identity>&) const;
+    void getReplicaGroups(std::set<std::string>&) const;
 
 private:
 
@@ -252,7 +264,7 @@ class NodeHelper
 {
 public:
 
-    NodeHelper(const std::string&, const NodeDescriptor&, const Resolver&);
+    NodeHelper(const std::string&, const NodeDescriptor&, const Resolver&, bool);
     virtual ~NodeHelper() { }
 
     bool operator==(const NodeHelper&) const;
@@ -262,9 +274,10 @@ public:
     NodeDescriptor update(const NodeUpdateDescriptor&, const Resolver&) const;
 
     void getIds(std::multiset<std::string>&, std::multiset<std::string>&, std::multiset<Ice::Identity>&) const;
+    void getReplicaGroups(std::set<std::string>&) const;
     const NodeDescriptor& getDefinition() const;
     const NodeDescriptor& getInstance() const;
-    void getServerInfos(const std::string&, std::map<std::string, ServerInfo>&) const;
+    void getServerInfos(const std::string&, const std::string&, int, std::map<std::string, ServerInfo>&) const;
     bool hasDistributions(const std::string&) const;
     bool hasServers() const;
     bool hasServer(const std::string&) const;
@@ -278,6 +291,7 @@ private:
     std::string _name;
     NodeDescriptor _def;    
     NodeDescriptor _instance;
+    bool _instantiated;
 
     typedef std::map<std::string, ServerInstanceHelper> ServerInstanceHelperDict;
     ServerInstanceHelperDict _serverInstances;
@@ -288,20 +302,21 @@ class ApplicationHelper
 {
 public:
 
-    ApplicationHelper(const Ice::CommunicatorPtr&, const ApplicationDescriptor&);
+    ApplicationHelper(const Ice::CommunicatorPtr&, const ApplicationDescriptor&, bool = false, bool = true);
 
     ApplicationUpdateDescriptor diff(const ApplicationHelper&) const;
     ApplicationDescriptor update(const ApplicationUpdateDescriptor&) const;
     ApplicationDescriptor instantiateServer(const std::string&, const ServerInstanceDescriptor&) const;
 
     void getIds(std::set<std::string>&, std::set<std::string>&, std::set<Ice::Identity>&) const;
+    void getReplicaGroups(std::set<std::string>&, std::set<std::string>&) const;
     const ApplicationDescriptor& getDefinition() const;
     const ApplicationDescriptor& getInstance() const;
     void getDistributions(DistributionDescriptor&, std::vector<std::string>&,const std::string& = std::string()) const;
 
-    void print(IceUtil::Output&) const;
+    void print(IceUtil::Output&, const ApplicationInfo&) const;
     void printDiff(IceUtil::Output&, const ApplicationHelper&) const;
-    std::map<std::string, ServerInfo> getServerInfos() const;
+    std::map<std::string, ServerInfo> getServerInfos(const std::string&, int) const;
 
 private:
 
@@ -313,8 +328,8 @@ private:
     NodeHelperDict _nodes;
 };
 
-bool descriptorEqual(const Ice::CommunicatorPtr&, const ServerDescriptorPtr&, const ServerDescriptorPtr&);
-ServerHelperPtr createHelper(const Ice::CommunicatorPtr&, const ServerDescriptorPtr&);
+bool descriptorEqual(const ServerDescriptorPtr&, const ServerDescriptorPtr&);
+ServerHelperPtr createHelper(const ServerDescriptorPtr&);
 
 }
 
