@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,12 +11,13 @@
 #include <Ice/Ice.h>
 #include <Ice/Service.h>
 #include <IceGrid/RegistryI.h>
+#include <IceGrid/TraceLevels.h>
 #ifdef __BCPLUSPLUS__
 #  include <IceGrid/AdminSessionI.h>
 #  include <IceGrid/WaitQueue.h>
 #  include <IceGrid/ReapThread.h>
-#  include <IceGrid/TraceLevels.h>
 #  include <IceGrid/Database.h>
+#  include <IceGrid/WellKnownObjectsManager.h>
 #endif
 
 using namespace std;
@@ -32,9 +33,12 @@ public:
 
     RegistryService();
 
+    virtual bool shutdown();
+
 protected:
 
     virtual bool start(int, char*[]);
+    virtual void waitForShutdown();
     virtual bool stop();
     virtual CommunicatorPtr initializeCommunicator(int&, char*[], const InitializationData&);
 
@@ -51,6 +55,14 @@ RegistryService::RegistryService()
 {
 }
 
+bool
+RegistryService::shutdown()
+{
+    assert(_registry);
+    _registry->shutdown();
+    return true;
+}
+
 bool
 RegistryService::start(int argc, char* argv[])
 {
@@ -64,52 +76,68 @@ RegistryService::start(int argc, char* argv[])
     vector<string> args;
     try
     {
-    	args = opts.parse(argc, (const char**)argv);
+        args = opts.parse(argc, (const char**)argv);
     }
-    catch(const IceUtil::Options::BadOpt& e)
+    catch(const IceUtil::BadOptException& e)
     {
         error(e.reason);
-	usage(argv[0]);
-	return false;
+        usage(argv[0]);
+        return false;
     }
 
-    if(opts.isSet("h") || opts.isSet("help"))
+    if(opts.isSet("help"))
     {
-	usage(argv[0]);
-	return false;
+        usage(argv[0]);
+        return false;
     }
-    if(opts.isSet("v") || opts.isSet("version"))
+    if(opts.isSet("version"))
     {
-	print(ICE_STRING_VERSION);
-	return false;
+        print(ICE_STRING_VERSION);
+        return false;
     }
     nowarn = opts.isSet("nowarn");
 
     if(!args.empty())
     {
-	usage(argv[0]);
-	return false;
+        usage(argv[0]);
+        return false;
     }
 
+    Ice::PropertiesPtr properties = communicator()->getProperties();
+
     //
     // Warn the user that setting Ice.ThreadPool.Server isn't useful.
     //
-    if(!nowarn && communicator()->getProperties()->getPropertyAsIntWithDefault("Ice.ThreadPool.Server.Size", 0) > 0)
+    if(!nowarn && properties->getPropertyAsIntWithDefault("Ice.ThreadPool.Server.Size", 0) > 0)
     {
-	Warning out(communicator()->getLogger());
-	out << "setting `Ice.ThreadPool.Server.Size' is not useful,\n";
-	out << "you should set individual adapter thread pools instead.";
+        Warning out(communicator()->getLogger());
+        out << "setting `Ice.ThreadPool.Server.Size' is not useful, ";
+        out << "you should set individual adapter thread pools instead.";
     }
+
+    TraceLevelsPtr traceLevels = new TraceLevels(communicator(), "IceGrid.Registry");
     
-    _registry = new RegistryI(communicator());
+    _registry = new RegistryI(communicator(), traceLevels);
     if(!_registry->start(nowarn))
     {
-	return false;
+        return false;
     }
 
     return true;
 }
 
+void
+RegistryService::waitForShutdown()
+{
+    //
+    // Wait for the activator shutdown. Once the run method returns
+    // all the servers have been deactivated.
+    //
+    enableInterrupt();
+    _registry->waitForShutdown();
+    disableInterrupt();
+}
+
 bool
 RegistryService::stop()
 {
@@ -119,7 +147,7 @@ RegistryService::stop()
 
 CommunicatorPtr
 RegistryService::initializeCommunicator(int& argc, char* argv[], 
-					const InitializationData& initializationData)
+                                        const InitializationData& initializationData)
 {
     InitializationData initData = initializationData;
     initData.properties = createProperties(argc, argv, initData.properties);
@@ -138,31 +166,31 @@ void
 RegistryService::usage(const string& appName)
 {
     string options =
-	"Options:\n"
-	"-h, --help           Show this message.\n"
-	"-v, --version        Display the Ice version.\n"
-	"--nowarn             Don't print any security warnings.";
+        "Options:\n"
+        "-h, --help           Show this message.\n"
+        "-v, --version        Display the Ice version.\n"
+        "--nowarn             Don't print any security warnings.";
 #ifdef _WIN32
     if(checkSystem())
     {
         options.append(
-	"\n"
-	"\n"
-	"--service NAME       Run as the Windows service NAME.\n"
-	"\n"
-	"--install NAME [--display DISP] [--executable EXEC] [args]\n"
-	"                     Install as Windows service NAME. If DISP is\n"
-	"                     provided, use it as the display name,\n"
-	"                     otherwise NAME is used. If EXEC is provided,\n"
-	"                     use it as the service executable, otherwise\n"
-	"                     this executable is used. Any additional\n"
-	"                     arguments are passed unchanged to the\n"
-	"                     service at startup.\n"
-	"--uninstall NAME     Uninstall Windows service NAME.\n"
-	"--start NAME [args]  Start Windows service NAME. Any additional\n"
-	"                     arguments are passed unchanged to the\n"
-	"                     service.\n"
-	"--stop NAME          Stop Windows service NAME."
+        "\n"
+        "\n"
+        "--service NAME       Run as the Windows service NAME.\n"
+        "\n"
+        "--install NAME [--display DISP] [--executable EXEC] [args]\n"
+        "                     Install as Windows service NAME. If DISP is\n"
+        "                     provided, use it as the display name,\n"
+        "                     otherwise NAME is used. If EXEC is provided,\n"
+        "                     use it as the service executable, otherwise\n"
+        "                     this executable is used. Any additional\n"
+        "                     arguments are passed unchanged to the\n"
+        "                     service at startup.\n"
+        "--uninstall NAME     Uninstall Windows service NAME.\n"
+        "--start NAME [args]  Start Windows service NAME. Any additional\n"
+        "                     arguments are passed unchanged to the\n"
+        "                     service.\n"
+        "--stop NAME          Stop Windows service NAME."
         );
     }
 #else
@@ -170,8 +198,9 @@ RegistryService::usage(const string& appName)
         "\n"
         "\n"
         "--daemon             Run as a daemon.\n"
+        "--pidfile FILE       Write process ID into FILE.\n"
         "--noclose            Do not close open file descriptors.\n"
-        "--nochdir            Do not change the current working directory."
+        "--nochdir            Do not change the current working directory.\n"
     );
 #endif
     print("Usage: " + appName + " [options]\n" + options);
