@@ -1,17 +1,19 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <IceStorm/SubscriberFactory.h>
-#include <IceStorm/Flusher.h>
+#include <IceUtil/DisableWarnings.h>
 #include <IceStorm/TopicI.h>
 #include <IceStorm/TopicManagerI.h>
+#include <IceStorm/Instance.h>
 #include <IceStorm/TraceLevels.h>
+#include <IceStorm/BatchFlusher.h>
+#include <IceStorm/SubscriberPool.h>
 #include <IceStorm/Service.h>
 
 using namespace std;
@@ -30,15 +32,15 @@ public:
     virtual ~ServiceI();
 
     virtual void start(const string&,
-		       const CommunicatorPtr&,
-		       const StringSeq&);
+                       const CommunicatorPtr&,
+                       const StringSeq&);
 
     virtual void start(const CommunicatorPtr&, 
-		       const ObjectAdapterPtr&, 
-		       const ObjectAdapterPtr&,
-		       const string&, 
-		       const Ice::Identity&,
-		       const string&);
+                       const ObjectAdapterPtr&, 
+                       const ObjectAdapterPtr&,
+                       const string&, 
+                       const Ice::Identity&,
+                       const string&);
 
     virtual TopicManagerPrx getTopicManager() const;    
 
@@ -48,6 +50,7 @@ private:
 
     TopicManagerIPtr _manager;
     TopicManagerPrx _managerProxy;
+    InstancePtr _instance;
     ObjectAdapterPtr _topicAdapter;
     ObjectAdapterPtr _publishAdapter;
 };
@@ -57,7 +60,7 @@ private:
 extern "C"
 {
 
-ICE_STORM_SERVICE_API ::IceBox::Service*
+ICE_STORM_API ::IceBox::Service*
 createIceStorm(CommunicatorPtr communicator)
 {
     return new ServiceI;
@@ -67,11 +70,11 @@ createIceStorm(CommunicatorPtr communicator)
 
 ServicePtr
 IceStorm::Service::create(const CommunicatorPtr& communicator,
-			  const ObjectAdapterPtr& topicAdapter,
-			  const ObjectAdapterPtr& publishAdapter,
-			  const string& name,
-			  const Ice::Identity& id,
-			  const string& dbEnv)
+                          const ObjectAdapterPtr& topicAdapter,
+                          const ObjectAdapterPtr& publishAdapter,
+                          const string& name,
+                          const Ice::Identity& id,
+                          const string& dbEnv)
 {
     ServiceI* service = new ServiceI;
     ServicePtr svc = service;
@@ -88,43 +91,65 @@ IceStorm::ServiceI::~ServiceI()
 }
 
 void
-IceStorm::ServiceI::start(const string& name,
-			  const CommunicatorPtr& communicator,
-			  const StringSeq& args)
+IceStorm::ServiceI::start(
+    const string& name,
+    const CommunicatorPtr& communicator,
+    const StringSeq& args)
 {
     PropertiesPtr properties = communicator->getProperties();
-    TraceLevelsPtr traceLevels = new TraceLevels(name, properties, communicator->getLogger());
+
     _topicAdapter = communicator->createObjectAdapter(name + ".TopicManager");
     _publishAdapter = communicator->createObjectAdapter(name + ".Publish");
 
     //
     // We use the name of the service for the name of the database environment.
     //
-    Ice::Identity topicManagerId;
-    topicManagerId.category = properties->getPropertyWithDefault(name + ".InstanceName", "IceStorm");
+    string instanceName = properties->getPropertyWithDefault(name + ".InstanceName", "IceStorm");
+    Identity topicManagerId;
+    topicManagerId.category = instanceName;
     topicManagerId.name = "TopicManager";
-    _manager = new TopicManagerI(communicator, _topicAdapter, _publishAdapter, traceLevels, name, "topics");
-    _managerProxy = TopicManagerPrx::uncheckedCast(_topicAdapter->add(_manager, topicManagerId));
 
+    _instance = new Instance(instanceName, name, communicator, _publishAdapter);
+    
+    try
+    {
+        _manager = new TopicManagerI(_instance, _topicAdapter, name, "topics");
+        _managerProxy = TopicManagerPrx::uncheckedCast(_topicAdapter->add(_manager, topicManagerId));
+    }
+    catch(const Ice::Exception&)
+    {
+        _instance = 0;
+        throw;
+    }
+        
     _topicAdapter->activate();
     _publishAdapter->activate();
 }
 
 void
 IceStorm::ServiceI::start(const CommunicatorPtr& communicator,
-			  const ObjectAdapterPtr& topicAdapter,
-			  const ObjectAdapterPtr& publishAdapter,
-			  const string& name,
-			  const Ice::Identity& id,
-			  const string& dbEnv)
+                          const ObjectAdapterPtr& topicAdapter,
+                          const ObjectAdapterPtr& publishAdapter,
+                          const string& name,
+                          const Ice::Identity& id,
+                          const string& dbEnv)
 {
-    TraceLevelsPtr traceLevels = new TraceLevels(name, communicator->getProperties(), communicator->getLogger());
+    string instanceName = communicator->getProperties()->getPropertyWithDefault(name + ".InstanceName", "IceStorm");
+    _instance = new Instance(instanceName, name, communicator, publishAdapter);
 
     //
     // We use the name of the service for the name of the database environment.
     //
-    _manager = new TopicManagerI(communicator, topicAdapter, publishAdapter, traceLevels, dbEnv, "topics");
-    _managerProxy = TopicManagerPrx::uncheckedCast(topicAdapter->add(_manager, id));
+    try
+    {
+        _manager = new TopicManagerI(_instance, topicAdapter, dbEnv, "topics");
+        _managerProxy = TopicManagerPrx::uncheckedCast(topicAdapter->add(_manager, id));
+    }
+    catch(const Ice::Exception&)
+    {
+        _instance = 0;
+        throw;
+    }
 }
 
 TopicManagerPrx
@@ -138,13 +163,18 @@ IceStorm::ServiceI::stop()
 {
     if(_topicAdapter)
     {
-	_topicAdapter->deactivate();
+        _topicAdapter->destroy();
     }
     if(_publishAdapter)
     {
-	_publishAdapter->deactivate();
+        _publishAdapter->destroy();
     }
 
+    //
+    // Shutdown the instance.
+    //
+    _instance->shutdown();
+
     //
     // It's necessary to reap all destroyed topics on shutdown.
     //
