@@ -1,15 +1,16 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <Ice/Properties.h>
 #include <IceStorm/TraceLevels.h>
 
+#include <Ice/Properties.h>
+
 using namespace std;
 using namespace IceStorm;
 
@@ -20,6 +21,8 @@ TraceLevels::TraceLevels(const string name, const Ice::PropertiesPtr& properties
     topicCat("Topic"),
     flush(0),
     flushCat("Flush"),
+    subscriberPool(0),
+    subscriberPoolCat("SubscriberPool"),
     subscriber(0),
     subscriberCat("Subscriber"),
     logger(theLogger)
@@ -28,6 +31,7 @@ TraceLevels::TraceLevels(const string name, const Ice::PropertiesPtr& properties
     const_cast<int&>(topicMgr) = properties->getPropertyAsInt(keyBase + topicMgrCat);
     const_cast<int&>(topic) = properties->getPropertyAsInt(keyBase + topicCat);
     const_cast<int&>(flush) = properties->getPropertyAsInt(keyBase + flushCat);
+    const_cast<int&>(subscriberPool) = properties->getPropertyAsInt(keyBase + subscriberPoolCat);
     const_cast<int&>(subscriber) = properties->getPropertyAsInt(keyBase + subscriberCat);
 }
 
