@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -28,9 +28,13 @@ IceXML::ParserException::ParserException(const char* file, int line, const strin
 {
 }
 
+IceXML::ParserException::~ParserException() throw()
+{
+}
+
 const char* IceXML::ParserException::_name = "IceXML::ParserException";
 
-const string
+string
 IceXML::ParserException::ice_name() const
 {
     return _name;
