@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -369,7 +369,7 @@ usage(const char* n)
         "-DNAME=DEF           Define NAME as DEF.\n"
         "-UNAME               Remove any definition for NAME.\n"
         "-IDIR                Put DIR in the include file search path.\n"
-	"-E                   Print preprocessor output on stdout.\n"
+        "-E                   Print preprocessor output on stdout.\n"
         "--output-dir DIR     Create files in the directory DIR.\n"
         "-d, --debug          Print debug messages.\n"
         "--ice                Permit `Ice' prefix (for building Ice source code only)\n"
@@ -384,18 +384,6 @@ usage(const char* n)
 int
 main(int argc, char* argv[])
 {
-    string cppArgs;
-    vector<string> includePaths;
-    bool preprocess;
-    string output;
-    bool debug;
-    bool ice;
-    bool all;
-    bool noPackage;
-    bool checksum;
-    bool caseSensitive;
-    string prefix;
-
     IceUtil::Options opts;
     opts.addOpt("h", "help");
     opts.addOpt("v", "version");
@@ -417,62 +405,62 @@ main(int argc, char* argv[])
     {
         args = opts.parse(argc, (const char**)argv);
     }
-    catch(const IceUtil::Options::BadOpt& e)
+    catch(const IceUtil::BadOptException& e)
     {
-	cerr << argv[0] << ": " << e.reason << endl;
-	usage(argv[0]);
-	return EXIT_FAILURE;
+        cerr << argv[0] << ": " << e.reason << endl;
+        usage(argv[0]);
+        return EXIT_FAILURE;
     }
 
-    if(opts.isSet("h") || opts.isSet("help"))
+    if(opts.isSet("help"))
     {
-	usage(argv[0]);
-	return EXIT_SUCCESS;
+        usage(argv[0]);
+        return EXIT_SUCCESS;
     }
-    if(opts.isSet("v") || opts.isSet("version"))
+
+    if(opts.isSet("version"))
     {
-	cout << ICE_STRING_VERSION << endl;
-	return EXIT_SUCCESS;
+        cout << ICE_STRING_VERSION << endl;
+        return EXIT_SUCCESS;
     }
-    if(opts.isSet("D"))
+
+    string cppArgs;
+    vector<string> optargs = opts.argVec("D");
+    vector<string>::const_iterator i;
+    for(i = optargs.begin(); i != optargs.end(); ++i)
     {
-	vector<string> optargs = opts.argVec("D");
-	for(vector<string>::const_iterator i = optargs.begin(); i != optargs.end(); ++i)
-	{
-	    cppArgs += " -D\"" + *i + "\"";
-	}
+        cppArgs += " -D" + Preprocessor::addQuotes(*i);
     }
-    if(opts.isSet("U"))
+
+    optargs = opts.argVec("U");
+    for(i = optargs.begin(); i != optargs.end(); ++i)
     {
-	vector<string> optargs = opts.argVec("U");
-	for(vector<string>::const_iterator i = optargs.begin(); i != optargs.end(); ++i)
-	{
-	    cppArgs += " -U\"" + *i + "\"";
-	}
+        cppArgs += " -U" + Preprocessor::addQuotes(*i);
     }
-    if(opts.isSet("I"))
+
+    vector<string> includePaths = opts.argVec("I");
+    for(i = includePaths.begin(); i != includePaths.end(); ++i)
     {
-	includePaths = opts.argVec("I");
-	for(vector<string>::const_iterator i = includePaths.begin(); i != includePaths.end(); ++i)
-	{
-	    cppArgs += " -I\"" + *i + "\"";
-	}
+	cppArgs += " -I" + Preprocessor::normalizeIncludePath(*i);
     }
-    preprocess = opts.isSet("E");
-    if(opts.isSet("output-dir"))
-    {
-	output = opts.optArg("output-dir");
-    }
-    debug = opts.isSet("d") || opts.isSet("debug");
-    ice = opts.isSet("ice");
-    all = opts.isSet("all");
-    noPackage = opts.isSet("no-package");
-    checksum = opts.isSet("checksum");
-    if(opts.isSet("prefix"))
-    {
-	prefix = opts.optArg("prefix");
-    }
-    caseSensitive = opts.isSet("case-sensitive");
+
+    bool preprocess = opts.isSet("E");
+
+    string output = opts.optArg("output-dir");
+
+    bool debug = opts.isSet("debug");
+
+    bool ice = opts.isSet("ice");
+
+    bool all = opts.isSet("all");
+
+    bool noPackage = opts.isSet("no-package");
+
+    bool checksum = opts.isSet("checksum");
+
+    string prefix = opts.optArg("prefix");
+
+    bool caseSensitive = opts.isSet("case-sensitive");
 
     if(args.empty())
     {
@@ -483,7 +471,7 @@ main(int argc, char* argv[])
 
     int status = EXIT_SUCCESS;
 
-    for(vector<string>::const_iterator i = args.begin(); i != args.end(); ++i)
+    for(i = args.begin(); i != args.end(); ++i)
     {
         Preprocessor icecpp(argv[0], *i, cppArgs);
         FILE* cppHandle = icecpp.preprocess(false);
@@ -493,87 +481,86 @@ main(int argc, char* argv[])
             return EXIT_FAILURE;
         }
 
-	if(preprocess)
-	{
-	    char buf[4096];
-	    while(fgets(buf, static_cast<int>(sizeof(buf)), cppHandle) != NULL)
-	    {
-		if(fputs(buf, stdout) == EOF)
-		{
-		    return EXIT_FAILURE;
-		}
-	    }
-	    if(!icecpp.close())
-	    {
-		return EXIT_FAILURE;
-	    }
-	}
-	else
-	{
-	    UnitPtr u = Unit::createUnit(false, all, ice, caseSensitive);
-	    int parseStatus = u->parse(cppHandle, debug);
+        if(preprocess)
+        {
+            char buf[4096];
+            while(fgets(buf, static_cast<int>(sizeof(buf)), cppHandle) != NULL)
+            {
+                if(fputs(buf, stdout) == EOF)
+                {
+                    return EXIT_FAILURE;
+                }
+            }
+            if(!icecpp.close())
+            {
+                return EXIT_FAILURE;
+            }
+        }
+        else
+        {
+            UnitPtr u = Unit::createUnit(false, all, ice, caseSensitive);
+            int parseStatus = u->parse(cppHandle, debug);
 
-	    if(!icecpp.close())
-	    {
-		u->destroy();
-		return EXIT_FAILURE;
-	    }
+            if(!icecpp.close())
+            {
+                u->destroy();
+                return EXIT_FAILURE;
+            }
 
-	    if(parseStatus == EXIT_FAILURE)
-	    {
-		status = EXIT_FAILURE;
-	    }
-	    else
-	    {
-		string base = icecpp.getBaseName();
-		string::size_type pos = base.find_last_of("/\\");
-		if(pos != string::npos)
-		{
-		    base.erase(0, pos + 1);
-		}
+            if(parseStatus == EXIT_FAILURE)
+            {
+                status = EXIT_FAILURE;
+            }
+            else
+            {
+                string base = icecpp.getBaseName();
+                string::size_type pos = base.find_last_of("/\\");
+                if(pos != string::npos)
+                {
+                    base.erase(0, pos + 1);
+                }
 
-		//
-		// Append the suffix "_ice" to the filename in order to avoid any conflicts
-		// with Slice module names. For example, if the file Test.ice defines a
-		// Slice module named "Test", then we couldn't create a Python package named
-		// "Test" and also call the generated file "Test.py".
-		//
-		string file = prefix + base + "_ice.py";
-		if(!output.empty())
-		{
-		    file = output + '/' + file;
-		}
+                //
+                // Append the suffix "_ice" to the filename in order to avoid any conflicts
+                // with Slice module names. For example, if the file Test.ice defines a
+                // Slice module named "Test", then we couldn't create a Python package named
+                // "Test" and also call the generated file "Test.py".
+                //
+                string file = prefix + base + "_ice.py";
+                if(!output.empty())
+                {
+                    file = output + '/' + file;
+                }
 
-		IceUtil::Output out;
-		out.setUseTab(false);
-		out.open(file.c_str());
-		if(!out)
-		{
-		    cerr << argv[0] << ": can't open `" << file << "' for writing" << endl;
-		    u->destroy();
-		    return EXIT_FAILURE;
-		}
+                IceUtil::Output out;
+                out.open(file.c_str());
+                if(!out)
+                {
+                    cerr << argv[0] << ": can't open `" << file << "' for writing" << endl;
+                    u->destroy();
+                    return EXIT_FAILURE;
+                }
 
-		printHeader(out);
-		out << "\n# Generated from file `" << base << ".ice'\n";
+                printHeader(out);
+                out << "\n# Generated from file `" << base << ".ice'\n";
 
-		//
-		// Generate the Python mapping.
-		//
-		generate(u, all, checksum, includePaths, out);
+                //
+                // Generate the Python mapping.
+                //
+                generate(u, all, checksum, includePaths, out);
 
-		//
-		// Create or update the Python package hierarchy.
-		//
-		if(!noPackage)
-		{
-		    PackageVisitor visitor(argv[0], prefix + base + "_ice", output);
-		    u->visit(&visitor, false);
-		}
-	    }
+                //
+                // Create or update the Python package hierarchy.
+                //
+                if(!noPackage)
+                {
+                    PackageVisitor visitor(argv[0], prefix + base + "_ice", output);
+                    u->visit(&visitor, false);
+                }
+            }
 
-	    u->destroy();
-	}
+            u->destroy();
+        }
     }
 
     return status;
