@@ -1,13 +1,12 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#include <IceUtil/IceUtil.h>
 #include <Freeze/Freeze.h>
 #include <TestI.h>
 
@@ -21,16 +20,16 @@ class DelayedResponse : public Thread
 public:
 
     DelayedResponse(const Test::AMD_Servant_slowGetValuePtr& cb, int val) :
-	_cb(cb),
-	_val(val)
+        _cb(cb),
+        _val(val)
     {
     }
     
     virtual void
     run()
     {
-	ThreadControl::sleep(Time::milliSeconds(500));
-	_cb->ice_response(_val);
+        ThreadControl::sleep(Time::milliSeconds(500));
+        _cb->ice_response(_val);
     }
 
 private:
@@ -62,7 +61,7 @@ Test::ServantI::init(const RemoteEvictorIPtr& remoteEvictor, const Freeze::Evict
 Int
 Test::ServantI::getValue(const Current&) const
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     return value;
 }
 
@@ -70,16 +69,16 @@ Int
 Test::ServantI::slowGetValue(const Current&) const
 {
     IceUtil::ThreadControl::sleep(IceUtil::Time::seconds(1));
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     return value;
 }
 
 void
 Test::ServantI::slowGetValue_async(const AMD_Servant_slowGetValuePtr& cb,
-				   const Current&) const
+                                   const Current&) const
 {
     IceUtil::ThreadControl::sleep(IceUtil::Time::seconds(1));
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     (new DelayedResponse(cb, value))->start().detach();
 }
 
@@ -87,7 +86,7 @@ Test::ServantI::slowGetValue_async(const AMD_Servant_slowGetValuePtr& cb,
 void
 Test::ServantI::setValue(Int val, const Current&)
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     value = val;
 }
 
@@ -95,21 +94,26 @@ Test::ServantI::setValue(Int val, const Current&)
 void
 Test::ServantI::setValueAsync_async(const AMD_Servant_setValueAsyncPtr& __cb, Int value, const Current&)
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     _setValueAsyncCB = __cb;
     _setValueAsyncValue = value;
+    notify();
 }
 
 void
 Test::ServantI::releaseAsync(const Current& current) const
 {
-    if(_setValueAsyncCB)
+    Monitor<Mutex>::Lock sync(*this);
+    //
+    // Wait until the previous _async has been dispatched
+    //
+    while(_setValueAsyncCB == 0)
     {
-	Mutex::Lock sync(*this);
-        const_cast<Int&>(value) = _setValueAsyncValue;
-        _setValueAsyncCB->ice_response();
-        const_cast<AMD_Servant_setValueAsyncPtr&>(_setValueAsyncCB) = 0;
+        wait();
     }
+    const_cast<Int&>(value) = _setValueAsyncValue;
+    _setValueAsyncCB->ice_response();
+    const_cast<AMD_Servant_setValueAsyncPtr&>(_setValueAsyncCB) = 0;
 }
 
 void
@@ -119,11 +123,11 @@ Test::ServantI::addFacet(const string& name, const string& data, const Current&
 
     try
     {
-	_evictor->addFacet(facet, current.id, name);
+        _evictor->addFacet(facet, current.id, name);
     }
     catch(const Ice::AlreadyRegisteredException&)
     {
-	throw Test::AlreadyRegisteredException();
+        throw Test::AlreadyRegisteredException();
     }
 }
 
@@ -132,11 +136,11 @@ Test::ServantI::removeFacet(const string& name, const Current& current) const
 {
     try
     {
-	_evictor->removeFacet(current.id, name);
+        _evictor->removeFacet(current.id, name);
     }
     catch(const Ice::NotRegisteredException&)
     {
-	throw Test::NotRegisteredException();
+        throw Test::NotRegisteredException();
     }
 }
 
@@ -144,14 +148,14 @@ Test::ServantI::removeFacet(const string& name, const Current& current) const
 Ice::Int
 Test::ServantI::getTransientValue(const Current& current) const
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     return _transientValue;
 }
 
 void
 Test::ServantI::setTransientValue(Ice::Int val, const Current& current)
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     _transientValue = val;
 }
 
@@ -166,11 +170,11 @@ Test::ServantI::release(const Current& current)
 {
     try
     {
-	_evictor->release(current.id);
+        _evictor->release(current.id);
     }
     catch(const Ice::NotRegisteredException&)
     {
-	throw NotRegisteredException();
+        throw NotRegisteredException();
     }
 }
 
@@ -179,11 +183,11 @@ Test::ServantI::destroy(const Current& current)
 {
     try
     {
-	_evictor->remove(current.id);
+        _evictor->remove(current.id);
     }
     catch(const Ice::NotRegisteredException&)
     {
-	throw Ice::ObjectNotExistException(__FILE__, __LINE__);
+        throw Ice::ObjectNotExistException(__FILE__, __LINE__);
     }
 }
 
@@ -202,14 +206,14 @@ Test::FacetI::FacetI(const RemoteEvictorIPtr& remoteEvictor, const Freeze::Evict
 string
 Test::FacetI::getData(const Current&) const
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     return data;
 }
 
 void
 Test::FacetI::setData(const string& d, const Current&)
 {
-    Mutex::Lock sync(*this);
+    Monitor<Mutex>::Lock sync(*this);
     data = d;
 }
 
@@ -221,7 +225,7 @@ public:
     void init(const Test::RemoteEvictorIPtr& remoteEvictor, const Freeze::EvictorPtr& evictor)
     {
         _remoteEvictor = remoteEvictor;
-	_evictor = evictor;
+        _evictor = evictor;
     }
     
     virtual void
@@ -239,7 +243,7 @@ private:
 
 
 Test::RemoteEvictorI::RemoteEvictorI(const ObjectAdapterPtr& adapter, const string& envName,
-				     const string& category) :
+                                     const string& category) :
     _adapter(adapter),
     _category(category)
 {
@@ -271,19 +275,19 @@ Test::RemoteEvictorI::createServant(const string& id, Int value, const Current&)
     ServantPtr servant = new ServantI(this, _evictor, value);
     try
     {
-	return ServantPrx::uncheckedCast(_evictor->add(servant, ident));
+        return ServantPrx::uncheckedCast(_evictor->add(servant, ident));
     }
     catch(const Ice::AlreadyRegisteredException&)
     {
-	throw Test::AlreadyRegisteredException();
+        throw Test::AlreadyRegisteredException();
     }
     catch(const Ice::ObjectAdapterDeactivatedException&)
     {
-	throw EvictorDeactivatedException();
+        throw EvictorDeactivatedException();
     }
     catch(const Freeze::EvictorDeactivatedException&)
     {
-	throw EvictorDeactivatedException();
+        throw EvictorDeactivatedException();
     }
 
 }
@@ -307,8 +311,7 @@ Test::RemoteEvictorI::saveNow(const Current& current)
 void
 Test::RemoteEvictorI::deactivate(const Current& current)
 {
-    _evictorAdapter->deactivate();
-    _evictorAdapter->waitForDeactivate();
+    _evictorAdapter->destroy();
     _adapter->remove(_adapter->getCommunicator()->stringToIdentity(_category));
 }
 
@@ -324,7 +327,7 @@ Test::RemoteEvictorI::destroyAllServants(const string& facetName, const Current&
     Freeze::EvictorIteratorPtr p = _evictor->getIterator(facetName, batchSize);
     while(p->hasNext())
     {
-	_evictor->remove(p->next());
+        _evictor->remove(p->next());
     }
 }
 
@@ -341,7 +344,7 @@ Test::RemoteEvictorFactoryI::createEvictor(const string& name, const Current& cu
 {
     RemoteEvictorIPtr remoteEvictor = new RemoteEvictorI(_adapter, _envName, name);  
     return RemoteEvictorPrx::uncheckedCast(_adapter->add(remoteEvictor, 
-    						         _adapter->getCommunicator()->stringToIdentity(name)));
+                                                         _adapter->getCommunicator()->stringToIdentity(name)));
 }
 
 void
