@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -26,7 +26,8 @@ testdir = os.path.join(toplevel, "test", name)
 client = os.path.join(testdir, "client")
 
 print "starting client...",
-clientPipe = os.popen(client + TestUtil.clientOptions + " 2>&1")
+seedfile = testdir + "/seed"
+clientPipe = os.popen(client + TestUtil.clientOptions + " " + seedfile + " 2>&1")
 print "ok"
 
 TestUtil.printOutputFromPipe(clientPipe)
@@ -36,4 +37,6 @@ clientStatus = TestUtil.closePipe(clientPipe)
 if clientStatus:
     sys.exit(1)
 
+os.remove(seedfile)
+
 sys.exit(0)
