@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,9 +16,3 @@ TestI::TestI(const Ice::ObjectAdapterPtr& adapter, const Ice::PropertiesPtr& pro
 {
 }
 
-std::string
-TestI::getReplicaIdAndShutdown(const Ice::Current&)
-{
-    _adapter->getCommunicator()->shutdown();
-    return _properties->getProperty(_adapter->getName() + ".AdapterId");
-}
