@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -178,14 +178,14 @@ main(int, char**)
     badQuoteCommands.push_back("-Dir=$'test");
     for(vector<string>::const_iterator p = badQuoteCommands.begin(); p != badQuoteCommands.end(); ++p)
     {
-	try
-	{
-	    IceUtil::Options::split(*p);
-	    test(false);
-	}
-	catch(const IceUtil::Options::BadQuote&)
-	{
-	}
+        try
+        {
+            IceUtil::Options::split(*p);
+            test(false);
+        }
+        catch(const IceUtil::BadOptException&)
+        {
+        }
     }
     cout << "ok" << endl;
 
