@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,6 +13,10 @@
 #include <IceUtil/Config.h>
 #include <cstdlib>
 
+#ifdef __BCPLUSPLUS__
+#include <stdlib.h>
+#endif
+
 void
 inline testFailed(const char* expr, const char* file, unsigned int line)
 {
